<?php
/**
* 2007-2020 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
*
*	@author	IPPANEL
*	@copyright 2020 ippanel
*	@license	 http://opensource.org/licenses/afl-3.0.php	Academic Free License (AFL 3.0)
*/

if (!defined('_PS_VERSION_')) {
	exit;
}
class IppanelSms extends Module
{
	public $configs;
	/* تعریف متغیرهای اصلی و پرکاربرد */
	public function __construct()
	{
		$this->preload();
		$this->name			= 'ippanelsms';
		$this->tab			 = 'emailing';
		$this->version		 = '2.0.4';
		$this->author		= 'IPPANEL';
		$this->need_instance = 0;
		$this->bootstrap = true;
		parent::__construct();
		$this->displayName		= $this->l('Ippanel Sms Gateway');
		$this->description		= $this->l('Ippanel sms web service for PrestaShop');
		$this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
		$this->prefix			 = 'IPPANEL_SMS_';
		$context = Context::getContext();
		$this->shop_id = (int)$context->shop->id;
		$this->shop = Configuration::get($this->prefix .'SHOPNAME' , null , null , $this->shop_id);
		$this->enable = Configuration::get($this->prefix .'SMSENABLE' , null , null , $this->shop_id);
		$this->PSversion = Configuration::get('PS_INSTALL_VERSION');
		$this->domain = Configuration::get('PS_SHOP_DOMAIN');
		$this->server	= trim(Configuration::get($this->prefix .'PANELADDRESS' , null , null , $this->shop_id), "/ ");
		if(empty($this->server)) $this->server = 'https://ippanel.com';
		$this->cronpass = false;
		$this->folder = _PS_MODULE_DIR_.$this->name.'/';
		$this->patterns = @file_get_contents(__DIR__.'/include/patternsmshumanity.txt');
		$this->patternspliter = ':';
	}
	
	/* تعریف باکس اطلاعات پنل در داشبورد مدیریت */
	public function hookDashboardZoneOne($params)
	{
		$this->context->smarty->assign(
			array(
				'ajax_link' => $this->context->link->getAdminLink('ippanelAdminAjax'),
				'link' => $this->context->link
			)
		);
		return $this->display(__FILE__, 'dashboard_ippanel.tpl');
	}
	
	/* تعریف اطلاعات نمایش داده شده در باکس داشبورد */
	public function hookDashboardData($params)
	{
		$auth = $this->panelAuth();
		if (!class_exists('jDateTime')) {
			include_once('include/jDateTime.php');
		}
		if($auth['ok'] == 'user' && is_numeric($auth['expireTime'])){
			$expiredate = $auth['expireTime'] . ' ' . $this->l('Days');
		}else{
			$expiredate = $auth['expireTime'];
		}
		return array(
			'data_value' => array(
				'panel_credit' => ($auth['ok'] == 'user') ? $auth['credit'] . ' ' . $this->l('Rials') : 'تنظیم نشده',
				'panel_expire' => $expiredate,
				'update_time' => ($auth['ok'] == 'user') ? $this->timeago($auth['updateTime']) : '---',
			),
		);
	}
	
	/***
	 * نصب افزونه
	 */
	public function install()
	{
	$error = 0;
	if (Shop::isFeatureActive())
	{
		Shop::setContext(Shop::CONTEXT_ALL);
	}
		if (!extension_loaded('curl')) {
			return false;
		}
		if (!$this->installConfig()) {
			return false;
		}
		if (!$this->installDatabase()) {
			
			return false;
		}
		//($tabClass, $tabName, $idTabParent, $modulename)
		if(!parent::install()
			or !$this->installModuleTab($this->name, $this->displayName, (int)Tab::getIdFromClassName('CONFIGURE'),$this->name)
			or !$this->installModuleTab('ippanelAdminAjax', 'Ippanel Ajax Class', -1,$this->name)
		) return false;
		include_once('include/hooks.php');
		foreach($this->hooks as $hook){
			if (!$this->registerHook($hook)) $error = 1;
		}
		if($error == 1) return false; else return true;
	}

	/***
	 * حذف نصب افزونه
	 */
	public function uninstall()
	{
	if(Configuration::get($this->prefix .'UNINSTALLDB' , null , null , null) == '1'){
		if (!$this->unistallDatabase()) { 
			return false;
		} 
	}
		if (!$this->uninstallConfig()) {
			return false;
		}
		if (!parent::uninstall()) {
			return false;
		}
		if (!$this->uninstallModuleTab($this->name)) {
			return false;
		}
		if (!$this->uninstallModuleTab('ippanelAdminAjax')) {
			return false;
		}

		return true;
	}

	/***
	 * نصب تنظیمات افزونه 
	 */
	protected function installConfig()
	{
		include_once('include/configs.php');
	foreach($this->configs as $key => $value){
		if (!Configuration::updateValue($this->prefix . $key, $value)) return false;
		}
		return true;
	}

	/***
	 * حذف تنظیمات افزونه از دیتابیس 
	 */
	protected function uninstallConfig()
	{
		include_once('include/configs.php');
	foreach($this->configs as $key => $value){
		if (!Configuration::deleteByName($this->prefix . $key)) return false;
		}
		return true;
	}

	/***
	 * نصب جداول دیتابیس افزونه 
	 */
	public function installDatabase()
	{
		$sql = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . $this->name . '`(
				`smsid` INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
				`customer` INT(5) DEFAULT NULL ,
				`id_order` INT(10) DEFAULT NULL ,
				`phone` VARCHAR(100) DEFAULT NULL ,
				`position` VARCHAR(50) DEFAULT NULL ,
				`status` VARCHAR(15) NOT NULL ,
				`bulk` varchar(50) DEFAULT NULL,
				`delivery` varchar(50) DEFAULT NULL,
				`description` TEXT DEFAULT NULL,
				`timestamp` varchar(15) DEFAULT NULL,
				`type` varchar(10) DEFAULT NULL,
				`shop` varchar(10) DEFAULT NULL,
				`duration` varchar(10) DEFAULT NULL
				 )ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';
		$sql .= 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'ippanelsms_presmstext` (
			`id` int(11) NOT NULL AUTO_INCREMENT,
			`text` text COLLATE utf8_persian_ci NOT NULL,
			`weight` int(3) DEFAULT NULL,
			 PRIMARY KEY (`id`)
			 ) ENGINE=' . _MYSQL_ENGINE_ . ' AUTO_INCREMENT=1 DEFAULT CHARSET=utf8 COLLATE=utf8_persian_ci;';
		return Db::getInstance()->execute($sql);
	}
	
	/***
	 * حذف جداول دیتابیس افزونه 
	 */
	protected function unistallDatabase()
	{
		$sql = 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . $this->name .'`;';
		$sql .= 'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . $this->name . '_presmstext`;';
		return Db::getInstance()->execute($sql);
	}
	
	/**
	 * پاک کردن لاگ پیامک های ارسال شده.
	 *
	 */
	protected function emptyDatabase()
	{
		$sql = 'TRUNCATE TABLE `' . _DB_PREFIX_ . $this->name . '`';
		return Db::getInstance()->execute($sql);
	}
	
	/**
	 * نصب منوی افزونه.
	 */
	public function installModuleTab($tabClass, $tabName, $idTabParent, $modulename)
	{
			$tab = new Tab();
			$tab->active = 1;
			$tab->name[(int)Configuration::get('PS_LANG_DEFAULT')] = $tabName;
			$tab->class_name = $tabClass;
			$tab->module = $modulename;
			$tab->id_parent = $idTabParent;
			$tab->module = $this->name;
				if ($tab->save()) {
				return true;
			}
			return false;
	}

	/**
	 * حذف منوی افزونه.
	 */
	public function uninstallModuleTab($tabClass)
	{
		$id_tab = (int)Tab::getIdFromClassName($tabClass);
		if ($id_tab) {
			$tab = new Tab($id_tab);
			return $tab->delete();
		}
		return false;
	}
	
	/**
	 * مهیا سازی برای نمایش صفحه افزونه.
	 */
	public function preload()
	{
		if (Tools::getValue('controller') != '' && Tools::getValue('controller') == 'ippanelsms') {
			$token	 = Tools::getAdminTokenLite('AdminModules');
			$request_scheme = Tools::getShopProtocol();
			$hostlink = $request_scheme . $_SERVER['SERVER_NAME'] . $_SERVER['PHP_SELF'];
			$ctrlconfi = "?controller=AdminModules&configure=" . 'ippanelsms';
			$urlLink = $hostlink . $ctrlconfi ."&token=" . $token;
			Tools::redirect($urlLink);
		}
	}
	
	/**
	 * ایجاد لینک صفحات زیرمجموعه افزونه
	 */
	private function moduleSubpageUrl($subpage = null)
	{
		$token	 = Tools::getAdminTokenLite('AdminModules');
		$request_scheme = Tools::getShopProtocol();
		$hostlink = $request_scheme . $_SERVER['SERVER_NAME'] . $_SERVER['SCRIPT_NAME'];
		$ctrlconfi = "?controller=AdminModules&configure=" . $this->name;
		$urlLink = $hostlink . $ctrlconfi ."&token=" . $token;
		if($subpage) $urlLink .= "&subpage=" . $subpage;
		return $urlLink;
	}

	/**
	 * ایجاد محتوای صفحه تنظیمات افزونه.
	 */
	public function getContent()
	{	
		if (!extension_loaded('curl')) {
			return $this->displayError($this->l('cURL is not enabled. You should enable it before using thes module.'));
		}
		if (!$this->active) {
			return $this->displayError('افزونه فعال نیست. از صفحه ماژول ها فعال نمایید.');
		}
		$output = null;
		$time = time();
		$errors = array();
		include('include/submit.php');
		$auth = $this->panelAuth();
		$this->smarty->assign(array(
			'auth' => $auth,
			'alerttype' => 'info',
		));
		if(isset($this->context->cookie->redirect_errors)){
			$output .= $this->displayError($this->context->cookie->redirect_errors);
			// delete old messages
			unset($this->context->cookie->redirect_errors);
		}
		if(isset($this->context->cookie->redirect_success_message)){
			$output .= '<div class="alert alert-success"><button type="button" class="close" data-dismiss="alert">×</button>'.$this->context->cookie->redirect_success_message.'<div></div></div>';
			// delete old messages
			unset($this->context->cookie->redirect_success_message);
		}
		$output .= $this->display(__FILE__, 'views/templates/admin/headerinfo.tpl');

		$gatewayDescription = $gatewayWarning = $gatewaySuccess = $gatewayError = null;
		if ($auth['ok'] == 'user') {
			$sendnumber = str_replace('+98', '', Configuration::get($this->prefix .'SMSNUMBER' , null , null , $this->shop_id));
			$sendKhadamatiNumber = str_replace('+98', '', Configuration::get($this->prefix .'SMSSENUMBER' , null , null , $this->shop_id));
			if (in_array($sendnumber, $auth['lines'])) {
				$gatewaySuccess = $this->l('The gateway configured correctly!').' '
					.$this->l('Your remain credit is: ') .' '
					.$auth['credit'].' '
					.$this->l('rials').', '
					.$this->l('And your account expire time is after').' '
					.$auth['expireTime'].' '
					.$this->l('days');
				if($auth['updateTime']) $gatewaySuccess .= ' ('.$this->l('Panel info check time:').' '.$this->timeago($auth['updateTime']).')';
				if($auth['expireTime'] < 30) $gatewaySuccess .= '<br>' . $this->l('Please contact to SMS Service to extend panel.');
				if($auth['credit'] < 4000) $gatewaySuccess .= '<br>'.$this->l('Your credit is low.'). $this->l('Please go to your account and charge it.');
			} else {
				$gatewayWarning .= $this->l('The user and pass are correct but this line is not belong to you: ').$sendnumber.'.';
			}
			if ($sendKhadamatiNumber != '' and
				$sendKhadamatiNumber != null and
				!in_array($sendKhadamatiNumber, $auth['lines'])) {
				$gatewayWarning .= $this->l(' Service line is not belong to you: ').$sendKhadamatiNumber.'.';
			}
		} else {
			$gatewayError = $auth['error'];
		}
		if (version_compare(_PS_VERSION_, '1.6.1', '>=') === false) {
			$gatewayDescription= $gatewayDescription.$gatewayWarning.$gatewaySuccess.$gatewayError;
		}

		$helper = new HelperForm();
		$helper->module = $this;
		$helper->name_controller = $this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->show_toolbar = false;
		$helper->toolbar_scroll = false;
		$helper->table = $this->table;
		$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
		$helper->default_form_language = $lang->id;
		$allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG');
		$helper->allow_employee_form_lang = $allow_employee_form_lang ? $allow_employee_form_lang : 0;
		$this->fields_form = array();
		$helper->submit_action = $this->name;
		$helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).
		'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
		$helper->tpl_vars = array(
			'fields_value' => $this->getConfigFieldsValues(),
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id
		);
		if($auth['ok'] == 'user') $disabled = 0; else $disabled = 1;
		if($submited != '') {
			$activePage = $submited;
		} else {
			$activePage = Tools::getValue('subpage') ? Tools::getValue('subpage') : 'welcome';
		}
		$output .= "<div class='row ". $this->name ."settings'>";
		$output .= $this->moduleTabs($activePage);
		$output .= "<div class='col-lg-10 col-sm-9'>";
		$ippanelmodal = '
	<div class="modal fade" id="ippanelModal">
		<div class="modal-dialog">
			<div class="modal-content">
				<div class="modal-header">
					<button type="button" class="close modal-clear" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">×</span></button>
					<h4 class="modal-title"></h4>
				</div>
				<div class="modal-body">
					<center><span class="loader"></span></center>
				</div>
				<div class="modal-footer">
					<button type="button" class="btn btn-default modal-clear" data-dismiss="modal">'.$this->l('Close').'</button>
				</div>
			</div><!-- /.modal-content -->
		</div><!-- /.modal-dialog -->
	</div><!-- /.modal -->';
		switch($activePage){
			case 'gatewaysettings':
				include_once('include/gatewayform.php');
				$output .= $helper->generateForm(array($fields_form_gateway));
				break;
			case 'alertssettings':
				include_once('include/alertsform.php');
				$output .= $helper->generateForm(array($fields_form_alerts));
				break;
			case 'sendsms':
				if($disabled == 1) {
					$output .= '<div class="panel">	<div class="panel-heading"><i class="icon-paper-plane"></i>'.$this->l('Send SMS').'</div><div class="content">	<p>'.$this->l('Please config SMS Gateway first.').'</p></div></div>';
					break;
				}
				include_once('include/sendsmsform.php');
				$output .= $helper->generateForm(array($fields_form_send_sms));
				$output .= "<script>$('#PREPAREDTEXT').on('change', function(){if(this.value != '0') $('#SENDONETEXT').val(this.value);});</script>";
				break;
			case 'sendtoall':
				if($disabled == 1) {
					$output .= '<div class="panel">	<div class="panel-heading"><i class="icon-paper-plane"></i>'.$this->l('Send sms to customers').'</div><div class="content">	<p>'.$this->l('Please config SMS Gateway first.').'</p></div></div>';
					break;
				}
				include_once('include/sendtoallform.php');
				$output .= $helper->generateForm(array($fields_form_send_all_customers));
				break;
			case 'smstexts':
				include_once('include/smstextform.php');
				$output .= $helper->generateForm(array($fields_form_sms_texts));
				$output .= $ippanelmodal;
				break;
			case 'preparedsms':
				include_once('include/preparedsms.php');
				break;
			case 'sentsms':
				$output .= $this->logsPage();
				break;

			case 'testsystem':
				include_once('include/testsystem.php');
				break;
			case 'api':
				$api = new IppanelSmsApi;
				$output .= '<div class="panel" id="fieldset_welcome"><div class="panel-heading"><i class="icon-code"></i>'.$this->l('Api Document').'</div><div class="content">'.$api->help.'</div></div></div>';
				break;

			case 'welcome':
			default:
				include_once('include/welcomepage.php');
				$output .= $welcomePage;
				break;			
		}
		$output .="</div>";
		$output .="</div>";
		return $output;
	}
	
	/**
	 * ایجاد منوی خود افزونه
	 */
	private function moduleTabs($activePage){
		$pagesArray = array(
			'welcome' => $this->l('Welcome'),
			'gatewaysettings' => $this->l('Gateway Settings'),
			'alertssettings' => $this->l('Alerts Settings'),
			'smstexts' => $this->l('SMS Texts'),
			'preparedsms' => $this->l('Prepared SMS'),
			'sendsms' => $this->l('Send SMS'),
			'sendtoall' => $this->l('Send to All'),
			'sentsms' => $this->l('Sent SMS'),
			'testsystem' => $this->l('Test System'),
			'api' => $this->l('Api Document'),
		);
		$output = "<div class='ippanelsmstabs col-lg-2 col-sm-3'><div class='list-group'>";
		foreach($pagesArray as $key => $value){
			if($activePage == $key) {
				$class = 'class="'.$key.' list-group-item active"';
				$href = $this->moduleSubpageUrl($key);
			} else {
				$class = 'class="'.$key.' list-group-item"';
				$href = $this->moduleSubpageUrl($key);
			}
			
			$output .= "<a href='$href' $class>$value</a>";
		}
		$output .= '</div></div>';
		return $output;
	}

	/**
	 * گرفتن مقادیر متغیرهای صفحات افزونه
	 */
	public function getConfigFieldsValues()
	{
		include_once('include/configs.php');
		$output = array();
		foreach($this->configs as $key => $value){
			$output[$key] = Tools::getValue($key, Configuration::get($this->prefix . $key) , null , null , $this->shop_id);
		}
		$output['RECIVER'] = Tools::getValue('RECIVER');
		$output['SENDONETEXT'] = Tools::getValue('SENDONETEXT');
		$output['SENDALLTEXT'] = Tools::getValue('SENDALLTEXT');
		$output['PREPAREDTEXT'] = '0';
		$output['preparedSmsText'] = '';
		$output['SENTOALLSENDERLINE'] = '500010707';
		$output['SENTOALLDELAYTIME'] = 15;
		$output['SENTOALLPERONCE'] = '';
		$output['SENTOALLBETWEENTIME'] = 30;
		return $output;
	}
	
	/**
	 * افزودن فایل استایل و مقادیر جاوااسکریپت به تمام صفحات مدیریتی
	 **/
	public function hookBackOfficeHeader()
	{
	if (version_compare(_PS_VERSION_, '1.7', '>') == true) {
		$predefendedsmstext = str_replace(array( '"' ), array( "'" ), $this->preparedSMSoptions());
	}else{
		$predefendedsmstext = str_replace(array( "\n", "\r", '"' ), array( "\\n", "\\r", "\\'" ), $this->preparedSMSoptions());
	}
		$alertcredit = $ippaneltopmenu = 0;
		$auth = $this->panelAuth();
		if(Configuration::get($this->prefix .'LOWCREDITALERT' , null , null , null) == '1' && $auth['ok'] == 'user' && $auth['credit'] < 10000){
				$alertcredit = 1;
		}
		if(Configuration::get($this->prefix .'TOPMENU' , null , null , null) == '1'){
				$ippaneltopmenu = 1;
		}
		$this->context->controller->addCSS($this->_path . 'views/css/icon.css');

		Media::addJsDef(
			array(
		'ippanelUri' => '//'.Configuration::get('PS_SHOP_DOMAIN_SSL').'/modules/'.$this->name.'/',
		'ippanelAdminPage' => $this->moduleSubpageUrl(),
		'updateOrderStatusSmsAlert' => Configuration::get($this->prefix .'UPDATEORDERC' , null , null , $this->shop_id),
		'updateOrderTrackingSmsAlert' => Configuration::get($this->prefix .'ORDERTRACKING' , null , null , $this->shop_id),
		'predefendedsmstext' => $predefendedsmstext,
		'ippanelsms_ajax_url' => $this->context->link->getAdminLink('ippanelAdminAjax'),
		'ippanellowcreditalert' => $alertcredit,
		'ippaneltopmenu' => $ippaneltopmenu,
		'patternsplitter' => $this->patternspliter,
				)
			);
	}



	
	/**
	 * ارسال پیامک در هنگام ثبت نام مشتری به مشتری و ادمین
	 **/
	public function hookActionCustomerAccountAdd($params,$test = null)
	{
		$testresult = '';
		if (!$this->enable) {
			if($test) return 'سیستم پیامک فعال نیست.';
			return true;
		}
		if (Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id) == '' or
			Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id) == '') {
		if($test) return 'تنظیمات درگاه به درستی وارد نشده است.';
			return true;
		}

		if (is_object($params['newCustomer'])) {
			$newCustomer = get_object_vars($params['newCustomer']);
		}
		$gender = new Gender((int)$newCustomer['id_gender'], $this->context->language->id);
		$vars = array(
			'{gender}' => ($gender->name == 'آقا') ? 'آقای' : $gender->name,
			'{firstname}' => $this->sanitizeURL($newCustomer['firstname']),
			'{lastname}' => $this->sanitizeURL($newCustomer['lastname']),
			'{shop_name}' => $this->shop,
			'{password}' => '',
			'{email}' => $newCustomer['email'],
		);
		if(isset($_POST['password'])){
			$vars['{password}'] = $_POST['password'];
		}elseif(isset($params['_POST']['passwd'])){
			$vars['{password}'] = $params['_POST']['passwd'];
		}elseif($newCustomer['email'] == 'test@test.com'){
			$vars['{password}'] = $newCustomer['password'];
		}
		if (Configuration::get($this->prefix . 'NEWCUSTOMERA' , null , null , $this->shop_id) != 0 and Configuration::get($this->prefix . 'ADMINPHONE' , null , null , $this->shop_id) !== '') {
			$AdminPhone = Configuration::get($this->prefix . 'ADMINPHONE' , null , null , $this->shop_id);
			if ($text = $this->generateSMStext($vars, 'newacount', 'admin', null ,$params, $AdminPhone)) {
				$smsResult = $this->sendOne($text, $AdminPhone, '-', '-', $this->l('New Customer'));
				if($test && $smsResult == 'sent') {
					$testresult .= 'پیامک به مدیر (' . $AdminPhone . ') ارسال شد.<br>'; 
				}elseif($test && $smsResult == 'queue'){
					$testresult .= 'پیامک به مدیر (' . $AdminPhone . ') در صف ارسال قرار گرفت.<br>'; 
				}elseif($test){
					$testresult .= 'ارسال به مدیر انجام شد اما از طرف سامانه پیامک نتیجه خطا داده شد.<br>'; 
				}
			}
		}elseif($test){
			$testresult .= 'هشدار به مدیر هنگام ثبت نام مشتری غیرفعال است و یا شماره همراه مدیر در صفحه تنظیمات درگاه ثبت نشده است.<br>';
		}
		if (Configuration::get($this->prefix.'NEWCUSTOMERC' , null , null , $this->shop_id) != 0) {
			if($test){
				$phone = $newCustomer['phone'];
			}elseif(isset($params['_POST']['psy-otp']) && Tools::getValue('username') != '' && is_numeric(Tools::getValue('username'))){ // در صورت استفاده از ورودیار
				$id_customer = $newCustomer['id'];
				$id_shop = $newCustomer['id_shop'];
				$id_shop_group = $newCustomer['id_shop_group'];
				$mobile = Tools::getValue('username');
				$phone = $mobile;
			}else{
				$id_address		= Address::getFirstCustomerAddressId($newCustomer['id']);
				$customerAddress = new AddressCore($id_address);
				$phone			 = $customerAddress->phone_mobile;
			}
			if ($phone and !Tools::strlen($phone) < 10) {
				if ($text = $this->generateSMStext($vars, 'newacount', 'customer', null , $params, $phone)) {
					$smsResult = $this->sendOne($text, $phone, $newCustomer['id'], '-', $this->l('New Customer'));
					if($test && $smsResult == 'sent') {
						$testresult .= 'پیامک به مشتری (' . $phone . ') ارسال شد.<br>'; 
					}elseif($test && $smsResult == 'queue'){
						$testresult .= 'پیامک به مشتری (' . $phone . ') در صف ارسال قرار گرفت.'; 
					}elseif($test){
						$testresult .= 'ارسال به مشتری انجام شد اما از طرف سامانه پیامک نتیجه خطا داده شد.'; 
					}
				}
			}
		}elseif($test){
			$testresult .= 'هشدار به مشتری هنگام ثبت نام غیرفعال است.<br>';
		}
		if($test) return $testresult;
		return true;
	}
	
	/**
	 * ارسال پیامک پس از ثبت سفارش به مشتری و مدیر
	 **/
	public function hookActionValidateOrder($params, $test = null)
	{
		
		$testresult = '';
		if (!$this->enable) {
			if($test) return 'سیستم پیامک فعال نیست.';
			return;
		}
		if (Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id) == '' or
			Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id) == '') {
			if($test) return 'تنظیمات درگاه به درستی وارد نشده است.';
				return;
		}

		$order	= $params['order'];
		$customer = $params['customer'];
		$currency = $params['currency'];
		$gender = new Gender((int)$customer->id_gender, $this->context->language->id);
		if($test) $carrier = $params['carrier']; else $carrier = new Carrier((int) $order->id_carrier);
		if($test){
			$products = $order->products;
		}else{
			$product_list=$order->getProducts();
			$products = '';
			foreach($product_list as $product){
				$products .= $product['product_name'].' * ' .$product['product_quantity']."\n";
			}
		}
		$cart_message = '';
		$messageArray = Message::getMessagesByOrderId($order->id);
		if(is_array($messageArray) && isset($messageArray[0]['message'])){
			$cart_message = $messageArray[0]['message'];
		}
		$vars = array(
			'{gender}' => ($gender->name == 'آقا') ? 'آقای' : $gender->name,
			'{firstname}' => $this->sanitizeURL($customer->firstname),
			'{lastname}' => $this->sanitizeURL($customer->lastname),
			'{email}' => $customer->email,
			'{order_id}' => $order->id,
			'{order_reference}' => $order->reference,
			'{shop_name}' => $this->shop,
			'{payment}' => $order->payment,
			'{currency}' => $currency->name,
			'{carrier}' => $carrier->name,
			'{total_paid}' => round($order->total_paid),
			'{invoice}' => $this->sanitizeURL($this->invoiceShortLink($order->reference),'.'),
			'{adminInvoice}' => $this->adminInvoiceShortLink($order->reference),
			'{order_products}' => $products,
			'{cart_message}' => $cart_message,
		);
		if (Configuration::get($this->prefix . 'NEWORDERA' , null , null , $this->shop_id) != 0
			and Configuration::get($this->prefix . 'ADMINPHONE' , null , null , $this->shop_id) != '') {
			$adminSend = 1;
			if ($order->total_paid == 0 and Configuration::get($this->prefix . 'NEWORDERFA', null , null , $this->shop_id) == 0) {
				$adminSend = 0;
			}
			$AdminPhone = Configuration::get($this->prefix . 'ADMINPHONE' , null , null , $this->shop_id);
			if ($adminSend and $text = $this->generateSMStext($vars, 'neworder', 'admin', null , $params, $AdminPhone)) {
				$smsResult = $this->sendOne($text, $AdminPhone, '-', $order->id, $this->l('New Order'));
				if($test && $smsResult == 'sent') {
					$testresult .= 'پیامک به مدیر (' . $AdminPhone . ') ارسال شد.<br>'; 
				}elseif($test && $smsResult == 'queue'){
					$testresult .= 'پیامک به مدیر (' . $AdminPhone . ') در صف ارسال قرار گرفت.<br>'; 
				}elseif($test){
					$testresult .= 'ارسال به مدیر انجام شد اما از طرف سامانه پیامک نتیجه خطا داده شد.<br>';
				}
			}
		}elseif($test){
			$testresult .= 'هشدار به مدیر هنگام ثبت سفارش جدید غیرفعال است و یا شماره همراه مدیر در صفحه تنظیمات درگاه ثبت نشده است.<br>';
		}
		if (Configuration::get($this->prefix . 'NEWORDERC' , null , null , $this->shop_id) != 0) {
			$customaerSend = 1;
			if ($order->total_paid == 0 and Configuration::get($this->prefix . 'NEWORDERFC' , null , null , $this->shop_id) == 0) {
				$customaerSend = 0;
			}
			if($customaerSend){
				if($test){
					$phone = $customer->phone;
				}else{

					if($phone == ''){
						$orderMobileNumberType = Configuration::get($this->prefix . 'ORDERMOBILE' , null , null , $this->shop_id);
						if(!$orderMobileNumberType or $orderMobileNumberType == 'orderaddress'){
							$address = new Address(intval($order->id_address_invoice));
						}else{
							$id_address = Address::getFirstCustomerAddressId($customer->id);
							$address = new Address((int)($id_address));
						}
						if ($address->phone_mobile == null) {
							return;
						}
						$phone = $address->phone_mobile;
					}
				}
				if(Configuration::get('PSCA_EMAIL_GUEST') && Configuration::get('PSCA_EMAIL_GUEST') == $customer->email) $guest_email = Configuration::get('PSCA_EMAIL_GUEST'); else $guest_email = '';

				if ($phone and !Tools::strlen($phone) < 10 and $text = $this->generateSMStext($vars, 'neworder', 'customer', $guest_email , $params, $phone)) {
					$smsResult = $this->sendOne($text, $phone, $customer->id, $order->id, $this->l('New Order'));
					if($test && $smsResult == 'sent') {
						$testresult .= 'پیامک به مشتری (' . $phone . ') ارسال شد.<br>'; 
					}elseif($test && $smsResult == 'queue'){
						$testresult .= 'پیامک به مشتری (' . $phone . ') در صف ارسال قرار گرفت.'; 
					}elseif($test){
						$testresult .= 'ارسال به مشتری انجام شد اما از طرف سامانه پیامک نتیجه خطا داده شد.'; 
					}
				}
			}
		}elseif($test){
			$testresult .= 'هشدار به مشتری هنگام ثبت سفارش جدید غیرفعال است.<br>';
		}
		if($test) return $testresult;
		return;
	}
	
	/**
	 * ارسال پیامک هنگام تغییر وضعیت سفارش
	 **/
	public function hookActionOrderStatusPostUpdate($params, $test = null)
	{
		$customer_alert = Tools::getValue('customer_update_order_state_alert');
		if ((!$this->enable or $customer_alert != 'on') and $test == null ) {
			return;
		}
		if (Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id) == '' or
			Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id) == '') {
			if($test) return 'تنظیمات درگاه به درستی وارد نشده است.';
			return;
		}
		$cart_message = '';
		if($test){
			$order	= $params['order'];
			$customer = $params['customer'];
			$orderstate = new stdclass();
			$orderstate->name = $order->orderstate;
			$phone = $customer->phone;
			$carrier = $params['carrier'];
			$gender = new Gender((int)$customer->id_gender, $this->context->language->id);
			$products = $order->products;
		} else {
			$order	= new Order((int) ($params['id_order']));
			$orderstate = $params['newOrderStatus'];
			$customer = new Customer((int) $order->id_customer);
			$carrier = new Carrier((int) $order->id_carrier);
			$gender = new Gender((int)$customer->id_gender, $this->context->language->id);
			$phone = '';
			$product_list=$order->getProducts();
			$products = '';
			foreach($product_list as $product){
				$products .= $product['product_name'].' * ' .$product['product_quantity']."\n";
			}

			if($phone == ''){
				$orderMobileNumberType = Configuration::get($this->prefix . 'ORDERMOBILE' , null , null , $this->shop_id);
				if(!$orderMobileNumberType or $orderMobileNumberType == 'orderaddress'){
					$address = new Address(intval($order->id_address_invoice));
				}else{
					$id_address = Address::getFirstCustomerAddressId($customer->id);
					$address = new Address((int)($id_address));
				}
				if ($address->phone_mobile == null) {
					return;
				}
				$phone = $address->phone_mobile;
			}
			$messageArray = Message::getMessagesByOrderId($order->id);
			if(is_array($messageArray) && isset($messageArray[0]['message'])){
				$cart_message = $messageArray[0]['message'];
			}
		}
		$vars = array(
			'{gender}' => ($gender->name == 'آقا') ? 'آقای' : $gender->name,
			'{firstname}' => $this->sanitizeURL($customer->firstname),
			'{lastname}' => $this->sanitizeURL($customer->lastname),
			'{email}' => $customer->email,
			'{order_id}' => $order->id,
			'{order_reference}' => $order->reference,
			'{shop_name}' => $this->shop,
			'{order_state}' => $orderstate->name,
			'{carrier}' => $carrier->name,
			'{weekday}' => $this->_persianday(date("l",time())),
			'{total_paid}' => round($order->total_paid),
			'{invoice}' => $this->sanitizeURL($this->invoiceShortLink($order->reference),'.'),
			'{order_products}' => $products,
			'{cart_message}' => $cart_message,
		);

		if(Configuration::get('PSCA_EMAIL_GUEST') && Configuration::get('PSCA_EMAIL_GUEST') == $customer->email) $guest_email = Configuration::get('PSCA_EMAIL_GUEST'); else $guest_email = '';
		if (!$text = $this->generateSMStext($vars, 'updateorder', 'customer', $guest_email, $params, $phone)) {
			if($test) return 'اشکالی در ایجاد متن پیامک به وجود آمد.';
			return;
		}

		$smsResult = $this->sendOne($text, $phone, $customer->id, $order->id, $this->l('Update Order Status'));
		if($test && $smsResult == 'sent') {
			return 'پیامک به مشتری (' . $phone . ') ارسال شد.<br>'; 
		}elseif($test && $smsResult == 'queue'){
			return 'پیامک به مشتری (' . $phone . ') در صف ارسال قرار گرفت.'; 
		}elseif($test){
			return 'ارسال به مشتری انجام شد اما از طرف سامانه پیامک نتیجه خطا داده شد.'; 
		}
		if ($smsResult == 'sent') {
			$this->adminDisplayInformation($this->l('The sms has sent.'));
		} elseif ($smsResult == 'queue') {
			$this->adminDisplayWarning($this->l('The sms saved in queue and send at cron time'));
		} else {
			$this->adminDisplayWarning($this->l('The sms can\'t be sent. Error: ') . $smsResult);
		}
		return;
	}
	
	/**
	 *	برگرداندن نام فارسی روز برای متغیر weekday
	 **/
	private function _persianday($day){
			$days = array(
				'Saturday' => 'شنبه',
				'Sunday' => 'یکشنبه',
				'Monday' => 'دوشنبه',
				'Tuesday' => 'سه شنبه',
				'Wednesday' => 'چهارشنبه',
				'Thursday' => 'پنجشنبه',
				'Friday' => 'جمعه'
			);
		return $days[$day];
	}
	
	/**
	 * ارسال پیامک هنگام ثبت کد رهگیری محصول
	 **/	
	 public function hookactionAdminOrdersTrackingNumberUpdate($params , $test = null)
	{
		$customer_alert = Tools::getValue('customer_update_order_tracking_alert');
		if ((!$this->enable or $customer_alert != 'on') and $test == null ) {
			return true;
		}
		$order = $params['order'];
		$customer = $params['customer'];
		$carrier = $params['carrier'];
		$phone = '';
		$gender = new Gender((int)$customer->id_gender, $this->context->language->id);
		$orderObject = new Order((int) ($order->id));
		if($test){
			$products = $order->products;
		}else{
			$product_list=$orderObject->getProducts();
			$products = '';
			foreach($product_list as $product){
				$products .= $product['product_name'].' * ' .$product['product_quantity']."\n";
			}
		}
		$messageArray = Message::getMessagesByOrderId($order->id);
		if(is_array($messageArray) && isset($messageArray[0]['message'])){
			$cart_message = $messageArray[0]['message'];
		} else {
			$cart_message = '';
		}
		if($test){
			$phone = $customer->phone;
		}else{

			if($phone == ''){
				$orderMobileNumberType = Configuration::get($this->prefix . 'ORDERMOBILE' , null , null , $this->shop_id);
				if(!$orderMobileNumberType or $orderMobileNumberType == 'orderaddress'){
					$address = new Address(intval($order->id_address_invoice));
				}else{
					$id_address = Address::getFirstCustomerAddressId($customer->id);
					$address = new Address((int)($id_address));
				}

				if ($address->phone_mobile == null) {
					return true;
				}
				$phone = $address->phone_mobile;
			}
		}
		$vars = array(
			'{gender}' => ($gender->name == 'آقا') ? 'آقای' : $gender->name,
			'{firstname}' => $this->sanitizeURL($customer->firstname),
			'{lastname}' => $this->sanitizeURL($customer->lastname),
			'{email}' => $customer->email,
			'{order_id}' => $order->id,
			'{order_reference}' => $order->reference,
			'{shop_name}' => $this->shop,
			'{carrier}' => $carrier->name,
			'{tracking}' => $order->shipping_number,
			'{total_paid}' => round($order->total_paid),
			'{invoice}' => $this->sanitizeURL($this->invoiceShortLink($order->reference),'.'),
			'{order_products}' => $products,
			'{weekday}' => $this->_persianday(date("l",time())),
			'{cart_message}' => $cart_message,
		);
		
		if(Configuration::get('PSCA_EMAIL_GUEST') && Configuration::get('PSCA_EMAIL_GUEST') == $customer->email) $guest_email = Configuration::get('PSCA_EMAIL_GUEST'); else $guest_email = '';

		if (!$text = $this->generateSMStext($vars, 'updateOrderTracking', 'customer', $guest_email, $params,$phone)) {
			if($test) return 'اشکالی در ایجاد متن پیامک به وجود آمد.';
			return;
		}
		$smsResult = $this->sendOne($text, $phone, $customer->id, $order->id, $this->l('Update Tracking Number'));
		if($test && $smsResult == 'sent') {
			return 'پیامک به مشتری (' . $phone . ') ارسال شد.'; 
		}elseif($test && $smsResult == 'queue'){
			return 'پیامک به مشتری (' . $phone . ') در صف ارسال قرار گرفت.'; 
		}elseif($test){
			return 'ارسال به مشتری انجام شد اما از طرف سامانه پیامک نتیجه خطا داده شد.'; 
		}
		if ($smsResult == 'sent') {
			$this->adminDisplayInformation($this->l('The sms has sent.'));
		} elseif ($smsResult == 'queue') {
			$this->adminDisplayWarning($this->l('The sms saved in queue and send at cron time'));
		} else {
			$this->adminDisplayWarning($this->l('The sms can\'t be sent. Error: ') . $smsResult);
		}
		return;
	}

	/**
	 * ارسال پیامک هنگام ثبت آدرس جدید برای مشتری
	 **/
	public function hookActionObjectAddressAddAfter($params, $test = null)
	{
		if (Tools::getValue('controller') == 'AdminAddresses') {
			if(Tools::getValue('customer_add_address_alert') != 'on') return true;
		}
		if (!$this->enable) {
			if($test) return 'سیستم پیامک فعال نیست.';
			return true;
		}
		if (Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id) == '' or
			Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id) == '' or
			Configuration::get($this->prefix . 'NEWADDMOBILE' , null , null , $this->shop_id) == 0) {
			if($test) return 'تنظیمات درگاه به درستی وارد نشده است. یا هشدار در هنگام ثبت آدرس جدید فعال نیست.';
			return true;
		} 
		if (is_object($params['object'])) {

			$newAddress = get_object_vars($params['object']);

			if(isset($newAddress['id_gender'])){
			$gender = new Gender((int)$newAddress['id_gender'], $this->context->language->id);
			}else{
				$customer = new Customer((int) $newAddress['id_customer']);
				$gender = new Gender((int)$customer->id_gender, $this->context->language->id);
			}
			$vars = array(
				'{gender}' => ($gender->name == 'آقا') ? 'آقای' : $gender->name,
				'{firstname}' => $this->sanitizeURL($newAddress['firstname']),
				'{lastname}' => $this->sanitizeURL($newAddress['lastname']),
				'{shop_name}' => $this->shop,
				'{email}' => $customer->email,
			);
			$phone = $newAddress['phone_mobile'];
			if ($phone and !Tools::strlen($phone) < 10) {
				if ($text = $this->generateSMStext($vars, 'newaddress', 'customer', null, $params, $phone)) {
					$smsResult = $this->sendOne($text, $phone, $newAddress['id_customer'], '-', $this->l('New Address'));
					if($test && $smsResult == 'sent') {
						return 'پیامک به مشتری (' . $phone . ') ارسال شد.'; 
					}elseif($test && $smsResult == 'queue'){
						return 'پیامک به مشتری (' . $phone . ') در صف ارسال قرار گرفت.'; 
					}elseif($test){
						return 'ارسال به مشتری انجام شد اما از طرف سامانه پیامک نتیجه خطا داده شد.'; 
					}
				}
			}
		}
		return;
	} 

	/**
	 * ارسال پیامک کمبود موجودی برای مدیر
	 **/	
	public function hookactionUpdateQuantity($params, $test = null)
	{
		if (!$this->enable ) {
			if($test) return 'سیستم پیامک فعال نیست.';
			return true;
		}
		if (Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id) == '' or
			Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id) == '') {
			if($test) return 'تنظیمات درگاه به درستی وارد نشده است.';
			return true;
		}
		if($test){
			if($params['testtype'] == 'outofstock'){
				$isActivate = Configuration::get($this->prefix . 'OUTOFSTOCK' , null , null , $this->shop_id); 
				if ($isActivate != 0){
					$vars = array(
						'{product_name}' => $params['product_name'],
						'{product_id}' => $params['id_product'],
						'{shop_name}' => $this->shop,
						'{product_quantity}' => 0,
						'{product_name_only}' => $params['product_name_only'],
					);
					$adminPhone = Configuration::get($this->prefix . 'ADMINPHONE' , null , null , $this->shop_id);
					if (!$text = $this->generateSMStext($vars, 'outofstock', 'admin', null, $params,$adminPhone)) {
						return 'اشکالی در ایجاد متن پیامک به وجود آمد.';
					}
					$smsResult = $this->sendOne($text, $adminPhone, 'admin', '-', $this->l('Out of Stock'));
					if($test && $smsResult == 'sent') {
						return 'پیامک به مدیر (' . $adminPhone . ') ارسال شد.'; 
					}elseif($test && $smsResult == 'queue'){
						return 'پیامک به مدیر (' . $adminPhone . ') در صف ارسال قرار گرفت.'; 
					}elseif($test){
						return 'ارسال به مدیر انجام شد اما از طرف سامانه پیامک نتیجه خطا داده شد.'; 
					}
				}else{
					return 'ارسال به مدیر در هنگام اتمام موجودی کالا غیرفعال است.';
				}
			}
		} else{
			$id_product = (int)$params['id_product'];
			$id_product_attribute = (int)$params['id_product_attribute'];
			$context = Context::getContext();
			$id_shop = (int)$context->shop->id;
			$id_lang = (int)$context->language->id;
			$product = new Product($id_product, false, $id_lang, $id_shop, $context);
			if (empty($product)) {
				 return;
			}
			$product_name = Product::getProductName($id_product, $id_product_attribute, $id_lang);
			$product_name_only = Product::getProductName($id_product, null, $id_lang);
			$product_has_attributes = $product->hasAttributes();
			
			$check_oos = ($product_has_attributes && $id_product_attribute)
						|| (!$product_has_attributes && !$id_product_attribute);
			$product_quantity = (int)$params['quantity'];
			$adminPhone = Configuration::get($this->prefix . 'ADMINPHONE' , null , null , $this->shop_id);
			$isActivate = Configuration::get($this->prefix . 'OUTOFSTOCK' , null , null , $this->shop_id); 
			
			if ($isActivate != 0
			&& $adminPhone != ''
			&& $check_oos
			&& $product->active == 1
			&& $product_quantity <= Configuration::get($this->prefix . 'OUTOFSTOCKTHRESHOLD' , null , null , $this->shop_id)) {
				$vars = array(
					'{product_name}' => $product_name,
					'{product_name_only}' => $product_name_only,
					'{product_id}' => $id_product,
					'{shop_name}' => $this->shop,
					'{product_quantity}' => $product_quantity,
				);
				if (!$text = $this->generateSMStext($vars, 'outofstock', 'admin', null, $params, $adminPhone)) {
					return;
				}
				$this->sendOne($text, $adminPhone, 'admin', '-', $this->l('Out of Stock'));
			}
			return true;
		}
	}

	/**
	 * تبدیل پترن به متن عادی برای نمایش
	 **/
	private function patternSmsHumanity($text){
		$splited = explode(';', $text);
		$pattern_code_array = explode($this->patternspliter, $splited[0]);
		$pattern_code = $pattern_code_array[1];
		unset($splited[0]);
		$resArray = array();
		foreach ($splited as $parm) {
			$splited_parm = explode($this->patternspliter, $parm);
			$resArray[$splited_parm[0]] = $splited_parm[1];
		}		
		$patternsarray = array();
		$patternsmshumanity = $this->patterns;
		$patternsarray = json_decode($patternsmshumanity, true);
		if(isset($patternsarray[$pattern_code])){
			$patternMessage = $patternsarray[$pattern_code];
			preg_match_all('/\%\w*\%/m', $patternMessage, $matches);
		}else{
			$url = $this->server."/api/select";
			$param = array
				(
					'uname'=>Configuration::get($this->prefix . 'USERNAME' , null , null , null),
					'pass'=>Configuration::get($this->prefix . 'PASSWORD' , null , null , null),
					'patternCode'=>$pattern_code,
					'op'=>'getPatternParams'
				);

			$handler = curl_init($url);	
			curl_setopt($handler, CURLOPT_CONNECTTIMEOUT, 5); 
			curl_setopt($handler, CURLOPT_TIMEOUT, 20);			
			curl_setopt($handler, CURLOPT_CUSTOMREQUEST, "POST");
			curl_setopt($handler, CURLOPT_POSTFIELDS, json_encode($param));
			curl_setopt($handler, CURLOPT_RETURNTRANSFER, true);
			$result = curl_exec($handler);
			$response = json_decode($result,true);
			if (is_array($response) && isset($response['status']['code']) && $response['status']['code'] == 0) {
				$patternMessage = $response['data']['patternMessage'];
				$patternParams = $response['data']['patternParams'];
				preg_match_all('/\%\w*\%/m', $patternMessage, $matches);
				$patternsarray[$pattern_code] = $patternMessage;
				file_put_contents(__DIR__.'/include/patternsmshumanity.txt',json_encode($patternsarray));
			}else{
				return $text;
			}
		}
		foreach($resArray as $key => $value){
			$patternMessage = str_replace('%'.$key.'%', $value, $patternMessage);
		}
		$patternMessage = str_replace("\n", '<br>', $patternMessage);
		return $patternMessage;
	}
	/** 
	 * ایجاد متن پیامک برای رویدادهای مختلف
	 */
	private function generateSMStext($vars, $event, $reciver, $guest_email = null, $params = array(),$phone = '')
	{
		if(!isset($params['order']) && isset($params['id_order'])){
			$params['order'] = new Order((int) ($params['id_order']));
		}
		$serviceLineStatus = true;
		$sendServiceNumber = str_replace('+98', '', Configuration::get($this->prefix .'SMSSENUMBER' , null , null , $this->shop_id));
		if ($sendServiceNumber == '') {
			$serviceLineStatus = false;
		}
		if ($reciver == 'admin') {
			switch ($event) {
				case 'newacount':
					$type = Configuration::get($this->prefix .'NEWACADTETYPE' , null , null , $this->shop_id);
					if ($type == 'custom') {
						$massage = Configuration::get($this->prefix .'NEWACADTEXT' , null , null , $this->shop_id);
					} else {
						$massage = $this->l('{firstname} {lastname} registred on site.~{shop_name}');
					}
					break;
				case 'neworder':
					$type = Configuration::get($this->prefix .'NEWORADTETYPE' , null , null , $this->shop_id);
					if ($type == 'custom') {
						$massage = Configuration::get($this->prefix .'NEWORADTEXT' , null , null , $this->shop_id);
					} else {
						$massage = $this->l('New order at {shop_name}~Customer{firstname} {lastname}~Order: {order_id}~Payment: {payment}~Total: {total_paid} {currency}~{shop_name}');
					}
					break;
				case 'outofstock':
					$type = Configuration::get($this->prefix .'OUTOFSTOCKTETYPE' , null , null , $this->shop_id);
					if ($type == 'custom') {
						$massage = Configuration::get($this->prefix .'OUTOFSTOCKTEXT' , null , null , $this->shop_id);
					} else {
						$massage = $this->l('The product by id: {product_id} {product_name} goes out of stock.~{shop_name}');
					}
					break;
			}
		} else {
			switch ($event) {
				case 'newacount':
					$type = Configuration::get($this->prefix .'NEWACCUTETYPE' , null , null , $this->shop_id);
					if ($type == 'custom') {
						$massage = Configuration::get($this->prefix .'NEWACCUTEXT' , null , null , $this->shop_id);
					} else {
						$massage = $this->l('Dear {firstname} {lastname},~Welcome to our site.~{shop_name}');
					}
					break;
				case 'neworder':
					$type = Configuration::get($this->prefix .'NEWORCUTETYPE' , null , null , $this->shop_id);

					if ($type == 'custom') {
						$massage = Configuration::get($this->prefix .'NEWORCUTEXT' , null , null , $this->shop_id);
					} else {
						$massage = $this->l('Dear {firstname} {lastname},~Your order has created.~Order No: {order_id}~{shop_name}');
					}
					break;
				case 'updateorder':
					$type = Configuration::get($this->prefix .'UPORCUTETYPE' , null , null , $this->shop_id);

					if ($type == 'custom') {
						$massage = Configuration::get($this->prefix .'UPORCUTEXT' , null , null , $this->shop_id);
					} else {
						$massage = $this->l('Dear {firstname} {lastname},~New status for order: {order_id}: {order_state}~{shop_name}');
					}
					break;
				case 'updateOrderTracking':
					$type = Configuration::get($this->prefix .'UPORTRTETYPE' , null , null , $this->shop_id);

					if ($type == 'custom') {
						$massage = Configuration::get($this->prefix .'UPORTRTEXT' , null , null , $this->shop_id);
					} else {
						$massage = $this->l('Dear {firstname} {lastname},~Your order has sent by {carrier}~Order: {order_id}~Tracking No: {tracking}~{shop_name}');
					}
					break;
				case 'newaddress':
					$type = Configuration::get($this->prefix .'NEWADDRTETYPE' , null , null , $this->shop_id);
					if ($type == 'custom') {
						$massage = Configuration::get($this->prefix .'NEWADDRTEXT' , null , null , $this->shop_id);
					} else {
						$massage = $this->l('Dear {firstname} {lastname},~Welcome to our site.~{shop_name}');
					}
					break;

			}
		}
		//Add user custom vars
		preg_match_all("/{{.+}}/",$massage,$m);
		$post = $_POST;
		$get=$_GET;
		if(isset($m[0]) && is_array($m[0])){
			foreach($m[0] as $custom_var){
				$custom_var = str_replace(array('{{','}}'),array('',''),$custom_var);
				$customvar_array = explode('/',$custom_var);
				$new_custom_var = '';
				$value = ${$customvar_array[0]};
				unset($customvar_array[0]);
				foreach($customvar_array as $customvar_array_key){
					if (is_object($value)) {
						$value = get_object_vars($value);
					}
					 $value = $value[$customvar_array_key];
				}
				$vars['{{'.$custom_var.'}}'] = $value;
			}
		}
		if(Configuration::get($this->prefix .'DEBUG' , null , null , $this->shop_id) == 1){ //Export Availeble Variables to Debug File
			$debugoutput = '';
			$debug_post = $this->flatten($post);

			$debug_get = $this->flatten($get);
			if(!isset($params['order']) && isset($params['id_order'])){
				$params['order'] = new Order((int) ($params['id_order']));
			}
			$debug_params = $this->flatten($params);

			$debugoutput .= "Post Tokens:\n";
			foreach($debug_post as $key=>$value){
				$debugoutput .= '{{post/' . $key . '}} = ' . $value . "\n";
			}
			$debugoutput .= "\n\nGet Tokens:\n";
			foreach($debug_get as $key=>$value){
				$debugoutput .= '{{get/' . $key . '}} = ' . $value . "\n";
			}
			$debugoutput .= "\n\nParams Tokens:\n";
			foreach($debug_params as $key=>$value){
				$debugoutput .= '{{params/' . $key . '}} = ' . $value . "\n";
			}

			$this->debug($debugoutput,$event,true);
		}

		$massage = str_replace(array_keys($vars), array_values($vars), $massage);
		return $massage;
	}
	
	/**
	* ذخیره برخی اطلاعات در پوشه دیباگ افزونه هنگام ارسال پیامک
	**/
	public function debug($data,$file = 'ippanel_debug',$new = false){
		if(Configuration::get($this->prefix .'DEBUG' , null , null , $this->shop_id) == 1){
			if(is_array($data) || is_object($data)) $output = json_encode($data); else $output = $data;
			
			$file= __DIR__.'/debug/'.$file.'.txt';
			$debug_file_contents = @file_get_contents($file);
			if($new == false) $output = $debug_file_contents."\n".$output;
			file_put_contents($file,$output);
		}
	}
	
	/**
	* تبدیل آرایه ها به یک رشته جهت استفاده در ذخیره سازی آرایه ها در دیباگ
	**/
	private function flatten($array, $prefix = '') {
		$result = array();
		if(is_object($array)){
			$array = get_object_vars($array);
		}
		
		foreach($array as $key=>$value) {
			if(is_array($value) || is_object($value)) {
				$result = $result + $this->flatten($value, $prefix . $key . '/');
			}
			else {
				$result[$prefix . $key] = $value;
			}
		}
		return $result;
	}

	/**
	* افزودن متن پیامک پیش فرض
	**/
	private function addPreparedSMSText($text){
		$fields = array(
			'text' => pSQL($text),
		);
		return Db::getInstance()->insert($this->name.'_presmstext', $fields);
	}
	

	/**
	* فراخوانی تمام پیامک های پیش فرض
	**/
	public function getAllPreparedSMSText()
	{
		try{
			$sql = new DbQuery();
			$sql->select('*');
			$sql->from('ippanelsms_presmstext', 't');
			$sql->orderBy('t.weight ASC');
			return Db::getInstance()->executeS($sql);
		}

		//catch exception
		catch(Exception $e) {
			return array();
		}
	}

	/**
	* تغییر ترتیب پیامک های پیش فرض
	**/
	public function updatePreparedSMSTextSort($id,$weight)
	{
		Db::getInstance()->update('ippanelsms_presmstext', array('weight' => pSQL($weight)), 'id = ' . $id);
	}	
	/**
	* ایجاد لینک ارسال مجدد در صفحه لیست پیامک های ارسال شده
	**/
	public function displaySendagainLink($token, $id, $name)
	{

		return '<a class="sendagain" data-action="sendagain" title="'.$this->l('Send Again').'" data-logid="'.$id.'"><i class="icon-rocket"></i></a>';
	}
	/**
	* ذخیره ویرایش پیامک های پیش فرض
	**/	
	public function updatePreparedSMSTextMessage($id,$newText)
	{
		Db::getInstance()->update('ippanelsms_presmstext', array('text' => pSQL($newText)), 'id = ' . $id);
	}
	
	/**
	* حذف پیامک پیش فرض
	**/
	public function deletePreparedSMSTextMessage($id)
	{
		return Db::getInstance()->delete('ippanelsms_presmstext', '`id` = '.$id);
	}
	
	/**
	* ایجاد لیست پیامک های پیشفرض
	**/
	public function preparedSMSoptions()
	{
		$smstexts = $this->getAllPreparedSMSText();
		$output = '';
		if(count($smstexts)) {
			$output ='<select id="preparedsms" name="prepared" class="form-control"><option value="0">متن پیامک پیش فرض را انتخاب نمایید</option>';
			foreach($smstexts as $smstext){
				$output .='<option value="'.$smstext['text'].'" title="'.$smstext['text'].'">'.substr($smstext['text'],0,120).'...</option>';
			}
			$output .='</select>';
		}else{
			$output .='<div><a href="'.$this->moduleSubpageUrl('preparedsms').'">برای ثبت پیام های پیش فرض به صفحه مربوطه مراجعه نمایید.</a></div>';
		}
		
		return $output;
	}
	
	/** 
	 * ذخیره لاگ ارسال در دیتابیس
	 */
	public function saveLogs($status, $description, $bulk = null, $phone = null, $customer = null, $order = null, $position = null,$type = null,$duration = null)
	{	
		$time = time();
		$context = Context::getContext();
		$id_shop = (int)$context->shop->id;
		$fields = array(
			'customer' => pSQL($customer),
			'id_order' => pSQL($order),
			'phone' => pSQL($phone),
			'position' => pSQL($position),
			'status' => pSQL($status),
			'bulk' => pSQL($bulk),
			'description' => pSQL($description),
			'timestamp' => pSQL($time),
			'type' => pSQL($type),
			'shop' => pSQL($id_shop),
			'duration' => pSQL($duration),
		);
		if($type == 'bysample') $fields['delivery'] = 'bysample';
		if($status == 'failed') $fields['delivery'] = 'failed';
		if($status == 'queue') $fields['delivery'] = 'queue';
		if($status == 'notsendbyrule') $fields['delivery'] = 'notsendbyrule';
		return Db::getInstance()->insert($this->name, $fields);
	}

	/** 
	 * دریافت لاگ پیامک از دیتابیس
	 */
	public function getLogs($page = 0, $onpage = 25, $bulk = null, $phone = null, $customer = null, $order = null, $position = null, $shop_id = null, $status = null, $smsid = null, $description = null,$sentsmsOrderby = null, $sentsmsOrderway = null)
	{
		$timeago = time()-15552000; // 6 month
		if($onpage == 'cron') $onpage = Configuration::get($this->prefix . 'CRONITEMS' , null , null , $this->shop_id);
		$offset = $page * $onpage;
		$sql = new DbQuery();
		$sql->select('*');
		$sql->from($this->name, 't');
		if(!is_null($smsid))$sql->where("t.smsid = $smsid");
		if(!is_null($bulk))$sql->where("t.bulk = '{$bulk}'");
		if(!is_null($phone))$sql->where("t.phone like '%{$phone}%'");
		if(!is_null($customer))$sql->where("t.customer = '{$customer}'");
		if(!is_null($order))$sql->where("t.id_order = '{$order}'");
		if(!is_null($position))$sql->where("t.position like '%{$position}%'");
		if(!is_null($shop_id))$sql->where("t.shop = '{$shop_id}'");
		if(!is_null($status))$sql->where("t.status = '{$status}'");
		if(!is_null($description))$sql->where("t.description like '%{$description}%'");
		if(!is_null($sentsmsOrderby) && !is_null($sentsmsOrderway)){
			if($sentsmsOrderby == 'id') $sentsmsOrderby = 'smsid';
			if($sentsmsOrderby == 'time') $sentsmsOrderby = 'timestamp';
			$sql->orderBy($sentsmsOrderby.' '.$sentsmsOrderway);
		} else{
			$sql->orderBy('smsid DESC');
		}
		$sql->where("t.description like '%{$description}%'");
		$sql->where("t.timestamp > {$timeago}");
		if($onpage !== 'all') $sql->limit($onpage, $offset);
		return Db::getInstance()->executeS($sql);
	}


	/** 
	 * دریافت لاگ پیامک از دیتابیس مربوط به یک یا چند شماره خاص برای صفحه ویرایش مشتری
	 */
	public function getMobileLogs($page = 0, $onpage = 25, $bulk = null, $phone = null, $customer = null, $order = null, $position = null, $shop_id = null, $status = null, $smsid = null, $description = null,$sentsmsOrderby = null, $sentsmsOrderway = null)
	{
		$offset = $page * $onpage;
		$sql = new DbQuery();
		$sql->select('*');
		$sql->from($this->name, 't');
		if(!is_null($smsid))$sql->where("t.smsid = $smsid");
		if(!is_null($bulk))$sql->where("t.bulk = '{$bulk}'");
		if(!is_null($phone))$sql->where("t.phone IN ({$phone})");
		if(!is_null($customer))$sql->where("t.customer = '{$customer}'");
		if(!is_null($order))$sql->where("t.id_order = '{$order}'");
		if(!is_null($position))$sql->where("t.position like '%{$position}%'");
		if(!is_null($shop_id))$sql->where("t.shop = '{$shop_id}'");
		if(!is_null($status))$sql->where("t.status = '{$status}'");
		if(!is_null($description))$sql->where("t.description like '%{$description}%'");
		if(!is_null($sentsmsOrderby) && !is_null($sentsmsOrderway)){
			if($sentsmsOrderby == 'id') $sentsmsOrderby = 'smsid';
			if($sentsmsOrderby == 'time') $sentsmsOrderby = 'timestamp';
			$sql->orderBy($sentsmsOrderby.' '.$sentsmsOrderway);
		} else{
			$sql->orderBy('smsid DESC');
		}
		if(!is_null($description)) $sql->where("t.description like '%{$description}%'");
		if($onpage !== 'all') $sql->limit($onpage, $offset);
		return Db::getInstance()->executeS($sql);
	}
	
	/** 
	 *  دریافت تعداد لاگ  پیامک ها
	 */
	public function getLogsCount($page = 0, $onpage = 20, $bulk = null, $phone = null, $customer = null, $order = null, $position = null, $shop_id = null, $status = null)
	{
		$offset = $page * $onpage;
		$sql = new DbQuery();
		$sql->select('count(*) as cnt');
		$sql->from($this->name, 't');
		if(!is_null($bulk))$sql->where("t.bulk = '{$bulk}'");
		if(!is_null($phone))$sql->where("t.phone = '{$phone}'");
		if(!is_null($customer))$sql->where("t.customer = '{$customer}'");
		if(!is_null($order))$sql->where("t.order = '{$order}'");
		if(!is_null($position))$sql->where("t.position = '{$position}'");
		if(!is_null($shop_id))$sql->where("t.shop = '{$shop_id}'");
		if(!is_null($status))$sql->where("t.status = '{$status}'");
		return Db::getInstance()->executeS($sql);
	}
	
	/** 
	 * ویرایش لاگ پیامک
	 */
	public function updateLogs($bulk,$status)
	{
		Db::getInstance()->update($this->name, array('delivery' => pSQL($status)), 'bulk = ' . $bulk);
	}
	
	/** 
	 * حذف لاگ پیامک
	 */
	public function deleteLog($id)
	{
		return Db::getInstance()->delete('ippanelsms', '`smsid` = '.$id);
	}
	public function clearAllQueue()
	{
		Db::getInstance()->update($this->name, array('status' => 'canceled'), "status = 'queue'");
		Db::getInstance()->update($this->name, array('delivery' => 'canceled'), "delivery = 'queue'");
	}
	/** 
	 * دستور تهیه صفحه پیامک های ارسال شده
	 */
	public function logsPage($page = 0, $onpage = 20)
	{
		include_once('include/sentsms.php');
		return $output;
	}


	/**
	 * ترجمه وضعیت دلیوری پیامک های ارسال شده
	 */
	protected function deliveryStatus($logdelivery = null ,$bulk,$type){
		switch($logdelivery){

			case 'notsync':
			case 'send':
			case 'pending':
			case 'failed':
			case 'discarded':
			case 'deliveredTOGSM':
			case 'delivered':
			case 'queue':
			case 'canceled':
				return $this->deliveryStatusTranslate($logdelivery);
				break;
			default:
				if(!is_null($bulk)) {
					return '<span class="checkdelivery"><span class="bulk">'.$bulk.'</span><i class="process-icon-loading" title="در حال بررسی وضعیت ارسال"></i></span>';
				} else {
					return '';
				}
				break;
		}
	}
	
	/**
	 * ترجمه وضعیت ارسال پیامک
	 */
	public function deliveryStatusTranslate($status){
		if($status == null) return '---';
		$responseArray = array(
			'notsync' =>	 $this->l('Not Sync'),
			'send' =>		 $this->l('sent'),
			'pending' =>	 $this->l('pending'),
			'failed' =>	 $this->l('failed'),
			'discarded' =>	 $this->l('discarded'),
			'delivered' =>	 $this->l('delivered'),
			'deliveredTOGSM' =>	 $this->l('Delivered TO GSM'),
			'queue' =>	 $this->l('In Queue'),
			'canceled' =>	 $this->l('canceled'),
		);

		if(isset($responseArray[$status])) return $responseArray[$status]; else return $this->l('Not Defined:') . $status;
		
	}
	
	/**
	 * استخراج تمام شماره های یک مشتری
	 */
	protected function getPhoneMobiles($getTotal = false)
	{
		//get mobiles
		$mobilesArray = array();
		$mobiles = null;
		if (!$getTotal) {
			$sql = new DbQueryCore();
			$sql->select('DISTINCT phone_mobile');
			$sql->from('address');
			$query = $sql->build();

			$results=Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($query);
			if ($results and count($results)>0) {
				foreach ($results as $result) {
					if (!Tools::isEmpty($result['phone_mobile'])) {
						$mobilesArray[]=$result['phone_mobile'];
					}
				}
				$mobiles = implode(';', $mobilesArray);
			} else {
				return false;
			}
			return $mobiles;
		} else {
			$sql= new DbQueryCore();
			$sql->select('count(DISTINCT phone_mobile)');
			$sql->from('address');
			$sql->where('phone_mobile !=""');
			$query= $sql->build();
			return Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
		}
	}

	/** 
	* دریافت اطلاعات تاریخ انقضای پنل پیامک
	*/
	private function panelExpireTime()
	{
		$param = array('op'=>'usertime');
		$connectpanel = $this -> connectWebservice($param);
		if ($connectpanel && $time = strtotime($connectpanel)) {
			$now = new DateTime;
			$expire = new DateTime('@'.$time);
			$diff = $now->diff($expire);
			$remaindays = $diff->days;
		} else {
			$remaindays = 'بدون تاریخ انقضا';
		}
		return $remaindays;
	}

	/** 
	* بررسی کاربری پنل پیامک
	*/
	private function panelAuth()
	{
		if(Configuration::get($this->prefix . 'PANELAUTH' , null , null , $this->shop_id)){
			$authCache = unserialize(Configuration::get($this->prefix . 'PANELAUTH' , null , null , $this->shop_id));
			if($authCache['updateTime'] > time()-86400) return $authCache;		
		}
		$param = array('op'=>'credit');
		$connectpanel = $this -> connectWebservice($param);
		if (isset($connectpanel['status']) and $connectpanel['status'] == 'failed') {
			return array('ok' => false,'error' => $connectpanel['res_data']);
		}
		if ($connectpanel[0] == 0) {
			$authCache = array(
				'ok' => 'user',
				'credit' => round($connectpanel[1]),
				'expireTime' => $this -> panelExpireTime(),
				'lines' => $this -> panelLines(),
				'updateTime' => time(),
			);

			Configuration::updateValue($this->prefix . 'PANELAUTH' , serialize($authCache));
			return $authCache;
		} else {
			return array(
				'ok' => 'failed',
				'error' => $connectpanel[1],
			);
		}
	}

	/** 
	* دریافت خطوط در دسترس کاربر
	*/
	private function panelLines()
	{
		$param = array('op'=>'lines');
		$connectpanel = $this -> connectWebservice($param);
		return $connectpanel;
	}

	/**
	* دریافت مانده اعتبار پنل پیامک
	*/
	private function panelCredit()
	{
		$param = array('op'=>'credit');
		$connectpanel = $this -> connectWebservice($param);
		return round($connectpanel[1]);
	}


	/** 
	* ایجاد لینک کوتاه صورت حساب
	*/
	private function invoiceShortLink($orderReference = null)
	{
		if ($orderReference == null) {
			return '';
		}
		$shortInvoiceKey = Configuration::get($this->prefix . 'SHORTINVOICEKEY', null , null , $this->shop_id);
			$factorUrl = _PS_BASE_URL_ .'/'. $shortInvoiceKey.'/'.$orderReference ;
		return $factorUrl;
	}

	/**
	* ایجاد لینک کوتاه صورت حساب مدیر
	*/
	private function adminInvoiceShortLink($orderReference = null)
	{
		if ($orderReference == null) {
			return '';
		}
		$shortInvoiceKey = 'ipf';
			$factorUrl = _PS_BASE_URL_ .'/'. $shortInvoiceKey.'/'.$orderReference ;
		return $factorUrl;
	}

	/** 
	* ایجاد لینک کوتاه محصول
	*/
	public function productShortLink($productID = null)
	{
		if ($productID == null) {
			return '';
		}

		$shortProductKey = Configuration::get($this->prefix . 'SHORTPRODUCTKEY', null , null , $this->shop_id);
		$productUrl = _PS_BASE_URL_ .'/'. $shortProductKey.'/'.$productID ;
		return $productUrl;
	}

	/** 
	* دریافت دلیوری پیامک از پنل
	*/
	public function panelGetDelivery($uinqid)
	{
		$param = array('op'=>'delivery','uinqid'=>$uinqid);
		$connectpanel = $this -> connectWebservice($param);
		return $connectpanel;
	}

	/** 
	* قالب بندی زمان
	*/
	private function timeago($timestamp) {
		$strTime = array(
			$this->l('second'),
			$this->l('minute'),
			$this->l('hour'),
			$this->l('day'),
			$this->l('month'),
			$this->l('year')
		);
		$length = array("60","60","24","30","12","10");

		$currentTime = time();
		if($currentTime >= $timestamp) {
			$diff	 = time()- $timestamp;
			for($i = 0; $diff >= $length[$i] && $i < count($length)-1; $i++) {
				$diff = $diff / $length[$i];
			}
			$diff = round($diff);
			return $diff . ' ' . $strTime[$i] . ' ' . $this->l('(s) ago ');
		}
	}
	/** 
	* آماده سازی پیامک برای ارسال به پنل
	*/
	public function sendOne($text, $phonenumber, $customer = null, $order = null , $position = null)
	{
		$server = $this->server;
	if (!$this->enable) {
			return 'سیستم پیامک فعال نیست. به صفحه تنظیمات درگاه مراجعه نمایید.';
		}
		$text = trim($text);

		//If cron job is enabled:
		if(Configuration::get($this->prefix . 'CRONJOB' , null , null , $this->shop_id) && $this->cronpass == false){
			if($this->cronStates($position) == 1){
				$this->saveLogs('queue', $text, '?', $phonenumber, $customer, $order, $position,null,0);
				return 'queue'; 
			}
		}
		//If text is sample
	$text = str_replace('patterncode:','pcode:',strip_tags(trim($text)));
		if (Tools::substr($text, 0, 5) === "pcode") {
			$text = str_replace("\r\n",';',$text);
			$text = str_replace("\n",';',$text);
			$splited = explode(';', $text);
			$pattern_code = trim(explode($this->patternspliter, $splited[0])[1]);
			unset($splited[0]);
			$resArray = array();
			foreach ($splited as $parm) {
				$splited_parm = explode($this->patternspliter, $parm);
				$resArray[$splited_parm[0]] = trim($splited_parm[1]);
			}

			$user = Configuration::get($this->prefix . 'USERNAME' , null , null , $this->shop_id);
			$pass = Configuration::get($this->prefix . 'PASSWORD' , null , null , $this->shop_id);
			$fromNum = Configuration::get($this->prefix . 'SMSSENUMBER' , null , null , $this->shop_id);
			$toNum = explode(';', $phonenumber);
			$input_data = $resArray;
			foreach($toNum as $to){
				$url = $server."/patterns/pattern?username=".$user."&password=".urlencode($pass)."&from=".$fromNum."&to=".json_encode(array($to))."&input_data=".urlencode(json_encode($input_data))."&pattern_code=".$pattern_code;
				$starttime = time()+microtime();
				$handler = curl_init($url);			 
				curl_setopt($handler, CURLOPT_CUSTOMREQUEST, "GET");
				curl_setopt($handler, CURLOPT_RETURNTRANSFER, true);
				curl_setopt($handler, CURLOPT_CONNECTTIMEOUT, 10); 
				curl_setopt($handler, CURLOPT_TIMEOUT, 30); //timeout in seconds
				$response = curl_exec($handler);
				$response2 = json_decode($response);
				$endtime = time()+microtime();
				$duration = $endtime-$starttime;
				if (is_array($response2)) {
					$res_code = $response2[0];
					$res_data = $response2[1];
					$status = 'failed';
					$result = $this->getPanelErrors($res_code);
					$this->saveLogs($status, $text, '-', $phonenumber, $customer, $order, $position,'bysample',$duration);
				} else {
					$status = $result = 'sent';
					$this->saveLogs($status, $text, $response, $phonenumber, $customer, $order, $position,'bysample',$duration);
				}
			}
			return $result;
		}
		$rcpt_nm = explode(';', $phonenumber);
		$param = array();
		$param = array
					(
						'message'=> $text,
						'to'=>json_encode($rcpt_nm),
						'op'=> 'send'
					);
		if($position == $this->l('Send SMS Manual'))	{
			$param['from'] = Tools::getValue('SMSNUMBER');
		}elseif($position == $this->l('Send Customers'))	{
			$sender = Tools::getValue('SENTOALLSENDERLINE');
			$delay = Tools::getValue('SENTOALLDELAYTIME');
			$param['from'] = Tools::getValue('SENTOALLSENDERLINE');
			$param['time'] = date("Y-m-d H:i:s", time()+($delay*60));
		}elseif (count($rcpt_nm) > 5) {
			$param['time'] = date("Y-m-d H:i:s", time()+300);
			$phonenumber = 'multiple';
		}
		$starttime = time()+microtime();
		$connectpanel = $this -> connectWebservice($param);
		$endtime = time()+microtime();
		$duration = $endtime-$starttime;
		$status = $connectpanel['status'];
		$description = $text;
		$bulk = $connectpanel['res_data'];
		$this->saveLogs($status, $description, $bulk, $phonenumber, $customer, $order, $position,'normal',$duration);
		return $connectpanel['result'];
	}
	
	/**
	 *	لیست خطاهای وب سرویس
	 */
	private function getPanelErrors($error){
		$errorCodes = array(
			'0' => 'عملیات با موفقیت انجام شده است.',
			'1' => 'متن پیام خالی می باشد.',
			'2' => 'کاربر محدود گردیده است.',
			'3' => 'خط به شما تعلق ندارد.',
			'4' => 'گیرندگان خالی است.',
			'5' => 'اعتبار کافی نیست.',
			'7' => 'خط مورد نظر برای ارسال انبوه مناسب نمیباشد.',
			'9' => 'خط مورد نظر در این ساعت امکان ارسال ندارد.',
			'98' => 'حداکثر تعداد گیرنده رعایت نشدهه است.',
			'99' => 'اپراتور خط ارسالی قطع می باشد.',
			'21' => 'پسوند فایل صوتی نامعتبر است.',
			'22' => 'سایز فایل صوتی نامعتبر است.',
			'23' => 'تعداد تالش در پیام صوتی نامعتبر است.',
			'100' => 'شماره مخاطب دفترچه تلفن نامعتبر می باشد.',
			'101' => 'شماره مخاطب در دفترچه تلفن وجود دارد.',
			'102' => 'شماره مخاطب با موفقیت در دفترچه تلفن ذخیره گردید.',
			'111' => 'حداکثر تعداد گیرنده برای ارسال پیام صوتی رعایت نشده است.',
			'131' => 'تعداد تالش در پیام صوتی باید یکبار باشد.',
			'132' => 'آدرس فایل صوتی وارد نگردیده است.',
			'301' => 'از حرف ویژه در نام کاربری استفاده گردیده است.',
			'302' => 'قیمت گذاری انجام نگریدهه است.',
			'303' => 'نام کاربری وارد نگردیده است.',
			'304' => 'نام کاربری قبال انتخاب گردیده است.',
			'305' => 'نام کاربری وارد نگردیده است.',
			'306' => 'کد ملی وارد نگردیده است.',
			'307' => 'کد ملی به خطا وارد شده است.',
			'308' => 'شماره شناسنامه نا معتبر است.',
			'309' => 'شماره شناسنامه وارد نگردیده است.',
			'310' => 'ایمیل کاربر وارد نگردیده است.',
			'311' => 'شماره تلفن وارد نگردیده است.',
			'312' => 'تلفن به درستی وارد نگردیده است.',
			'313' => 'آدرس شما وارد نگردیده است.',
			'314' => 'شماره موبایل را وارد نکرده اید.',
			'315' => 'شماره موبایل به نادرستی وارد گردیده است.',
			'316' => 'سطح دسترسی به نادرستی وارد گردیده است.',
			'317' => 'کلمه عبور وارد نگردیده است.',
			'404' => 'پترن در دسترس نیست.',
			'455' => 'ارسال در آینده برای کد بالک ارسالی لغو شد.',
			'456' => 'کد بالک ارسالی نامعتبر است.',
			'458' => 'کد تیکت نامعتبر است.',
			'964' => 'شما دسترسی نمایندگی ندارید.',
			'962' => 'نام کاربری یا کلمه عبور نادرست می باشد.',
			'963' => 'دسترسی نامعتبر می باشد.',
			'971' => 'پترن ارسالی نامعتبر است.',
			'970' => 'پارامتر های ارسالی برای پترن نامعتبر است.',
			'972' => 'دریافت کننده برای ارسال پترن نامعتبر می باشد.',
			'992' => 'ارسال پیام از ساعت 8 تا 23 می باشد.',
			'993' => 'دفترچه تلفن باید یک آرایه باشد',
			'994' => 'لطفا تصویری از کارت بانکی خود را از منو مدارک ارسال کنید',
			'995' => 'جهت ارسال با خطوط اشتراکی سامانه، لطفا شماره کارت بانکیه خود را به دلیل تکمیل فرایند احراز هویت از بخش ارسال مدارک ثبت نمایید.',
			'996' => 'پترن فعال نیست.',
			'997' => 'شما اجازه ارسال از این پترن را ندارید.ه',
			'998' => 'کارت ملی یا کارت بانکی شما تایید نشده است.',
			'1001' => 'فرمت نام کاربری درست نمی باشد)حداقله ۵ کاراکتر، فقط حروف و اعداد(.',
			'1002' => 'گذر واژه خیلی ساده می باشد)حداقل ۸ کاراکتر بوده و نام کاربری،',
			'ایمی' => ' و شماره موبایل در آن وجود نداشته باشد(.',
			'1003' => 'مشکل در ثبت، با پشتیبانی تماس بگیرید.',
			'1004' => 'مشکل در ثبت، با پشتیبانی تماس بگیرید.',
			'1005' => 'مشکل در ثبت، با پشتیبانی تماس بگیرید.',
			'1006' => 'تاریخ ارسال پیام برای گذشته می باشد، لطفا تاریخ ارسال پیام را به درستی وارد نمایید.ه',
		);
		return (isset($errorCodes[$error])) ? $errorCodes[$error] : 'اشکال تعریف نشده با کد :' . $error;
	}
	
	/**
	 *	عنوان وضعیت های کرون
	 */
	private function cronStates($position){
		$states = array(
			'CRON_newacount' => $this->l('New Customer'),
			'CRON_neworder' => $this->l('New Order'),
			'CRON_updateorder' => $this->l('Update Order Status'),
			'CRON_updateOrderTracking' => $this->l('Update Tracking Number'),
			'CRON_newaddress' => $this->l('New Address'),
			'CRON_outofstock' => $this->l('Out of Stock'),
			'CRON_manual' => $this->l('Send SMS Manual'),
			'CRON_allcustomers' => $this->l('Send Customers'),
		);

		if(in_array($position,$states)){
			return Configuration::get($this->prefix . array_keys($states,$position)[0] , null , null , $this->shop_id);
		}
		return 0;
	}
	
	/** دستور عمومی برای ارسال پیامک از ماژول های دیگر
	* @param string $param['message']: sms massage
	* @param string $param['reciver']: recivers delimated by ;
	* @return boolean
	*/
	public function sendSMS($param = array() , $position = null)
	{
	$text = $param['message'];
	$phonenumber = $param['reciver'];
	if(isset($param['order_id'])) $order_id = $param['order_id']; else $order_id = '-';
	if(isset($param['customer_id'])) $customer_id = $param['customer_id']; else $customer_id = '-';
		$result = $this -> sendOne($text, $phonenumber, $customer_id, $order_id, $position);
		if ($result == 'sent' || $result == 'queue' || $result == 'notsendbyrule' ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	* ارتباط با پنل پیامک
	*/
	protected function connectWebservice($param = array())
	{
		$server = $this->server;
		$url = $server."/services.jspd";
		$param['uname'] = Configuration::get($this->prefix . 'USERNAME' , null , null , null);
		$param['pass'] = Configuration::get($this->prefix . 'PASSWORD' , null , null , null);
		if(!isset($param['from']) || $param['from'] == '') $param['from']= Configuration::get($this->prefix . 'SMSNUMBER' , null , null , null);

		if (isset($param['message'])) {
			$param['message'] = str_replace('~', "\n", $param['message']);
		}
		if (!$param['uname'] or !$param['pass'] or !$param['from']) {
			return array(
				'status' => 'failed',
				'result' => $this->l('Gateway configuration is not set proprely.'),
				'res_data' => $this->l('Gateway configuration is not set proprely.')
				);
		}
		$handler = curl_init($url);
		curl_setopt($handler, CURLOPT_CUSTOMREQUEST, "POST");
		curl_setopt($handler, CURLOPT_POSTFIELDS, $param);
		curl_setopt($handler, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($handler, CURLOPT_CONNECTTIMEOUT, 10); 
		curl_setopt($handler, CURLOPT_TIMEOUT, 30); //timeout in seconds
		$response = curl_exec($handler);
		$response2 = json_decode($response);
		$res_code = $response2[0];
		$res_data = $response2[1];
		switch ($param['op']) {
			case 'sendPattern':
				$result = array('status' => $res_code,'result' => $res_data);
				break;
			case 'usertime':
				$result = $res_data;
				break;
			case 'credit':
				$result = $response2;
				break;
			case 'delivery':
				if ($response == 'null') {
					return null;
				}
				if ($res_code == 0) {
					$result = json_decode($res_data);
					$res2 = explode(':', $result[0]);
					$res_data = $res2[1];
				}
				$result = $res_data;
				break;
			case 'lines':
				$linesArray = json_decode($res_data);
				$lines = array();
				if(count($linesArray)){
					foreach ($linesArray as $line) {
						$lines[] = str_replace('+98', '', json_decode($line)->number);
					}
				}
				$result = $lines;
				break;
			case 'send':

				if ($res_code == '0') {
					$status = $result = 'sent';
				} else {
					$status = 'failed';
					$result = $this->getPanelErrors($res_code);
				}
				$result = array('status' => $status,'result' => $result,'res_data' => $res_data);
				break;
		}
		return $result;
	}

	/**
	* افزودن فایل استایل و جاوااسکریپت به صفحات مدیریت
	*/
	public function hookActionAdminControllerSetMedia($params)
	{
		if (strtolower(Tools::getValue('controller')) == 'adminorders') { 
			$this->context->controller->addJS($this->_path.'views/js/adminorders.js');
		}
		if (Tools::getValue('controller') == 'AdminAddresses' && !Tools::getValue('id_address')) {
			$this->context->controller->addJS($this->_path.'views/js/adminaddress.js'); 
		}	
		if (strtolower(Tools::getValue('controller')) == 'admincustomers' and Tools::getValue('id_customer') && isset($_GET["viewcustomer"])) {
			$this->context->controller->addJS($this->_path.'views/js/admincustomers.js');
        } elseif (strtolower(Tools::getValue('controller')) == 'admincustomers' && isset($this->context->customer->id)) {
			$this->context->controller->addJS($this->_path.'views/js/admincustomers.js');
		} elseif (strtolower(Tools::getValue('controller')) == 'admincustomers') {
            //$this->context->controller->addJS($this->_path.'views/js/editcustomer.js');
		}
		if (Tools::getValue('controller') == 'AdminModules' and Tools::getValue('configure') == $this->name) {
			$this->context->controller->addCSS($this->_path.'views/css/backoffice.css', 'all');
			$this->context->controller->addJS($this->_path.'views/js/backoffice.js', 'all');
		}
		$this->context->controller->addCSS($this->_path.'views/css/backofficemobiles.css', 'all');
		$this->context->controller->addJS($this->_path.'views/js/backofficemobiles.js', 'all');
	}
	/**
	* نمایش کادر ارسال پیامک در صفحه ویرایش سفارش.
	*/
	public function hookDisplayAdminOrderLeft($params)
	{
	$order = new Order((int)$params['id_order']);
	$customer = new Customer((int) $order->id_customer);
	$address = new Address(intval($order->id_address_invoice));
	$gender = new Gender((int)$customer->id_gender, $this->context->language->id);
		$vars = array(
		'{gender}' => ($gender->name == 'آقا') ? 'آقای' : $gender->name,
		'{firstname}' => $this->sanitizeURL($customer->firstname),
		'{lastname}' => $this->sanitizeURL($customer->lastname),
		'{email}' => $customer->email,
		'{shop_name}' => $this->shop,
		'{order_id}' => $order->id,
		'{order_reference}' => $order->reference,
		'{payment}' => $order->payment,
		'{total_paid}' => round($order->total_paid),
		'{invoice}' => $this->sanitizeURL($this->invoiceShortLink($order->reference),'.'),
		);
		$preparedsmstexts = str_replace(array_keys($vars), array_values($vars), $this->preparedSMSoptions());
		$orderLastSms = $this->getMobileLogs(0, $onpage = 15, $bulk = null, null, null, $order->id);
		$orderLastSmsOutput = '';
		if (!class_exists('jDateTime')) {
			include_once('include/jDateTime.php');
		}
		foreach($orderLastSms as $sms){
			$orderLastSmsOutput .="<tr>
							<td>".$sms['bulk']."</td>
							<td>".$sms['phone']."</td>
							<td>".jDateTime::date('Y/m/d H:i:s', $sms['timestamp'], null, true,null)."</td>
							<td>".$sms['description']."</td>
							<td>".$this->l($sms['status'])."</td>
							<td>".$this->l($sms['delivery'])."</td>
						</tr>";
		}
		$this->smarty->assign(array(
			'order' => $order,
			'preparedsmstexts' => $preparedsmstexts,
			'phone' => $address->phone_mobile,
			'shop_id' => $this->shop_id,
			'customer_id' => $order->id_customer,
			'order_id' => $order->id,
			'orderLastSms' => $orderLastSmsOutput,
		));
		return $this->display(__FILE__, 'views/templates/admin/adminorder.tpl');
	}	
    /**
     * استخراج تمام آدرس های مربوط به یک مشتری.
     */
    public static function getAllCustomerAddresses($id_customer, $active = true)
    {

        if (!$id_customer) {
            return array();
        }

		$sql = new DbQuery();
		$sql->select('*');
		$sql->from('address', 'a');
		$sql->where("a.id_customer = ".(int) $id_customer);
		$sql->where("a.deleted = 0");
		if($active) $sql->where("a.active = 1");
		$adresses = Db::getInstance()->executeS($sql);

		return $adresses;
    }
	/**
	* نمایش کادر ارسال پیامک در صفحه ویرایش مشتری
	*/
	public function hookDisplayAdminCustomers($params)
	{

		$mobilephones =array();
		$allCustomerMobiles =array();
		$mobilesoptions = '';
		if(isset($params['smarty']->tpl_vars['addresses']->value)){
			$customer_addresses = $params['smarty']->tpl_vars['addresses']->value;
		}else{
			$customer_addresses = $this->getAllCustomerAddresses($params['id_customer'], true);
		}
		foreach($customer_addresses as $address){
			
			if(!isset($mobilephones[$address['phone_mobile']])){
				$mobilesoptions .= '<option value="'.$address['phone_mobile'].'">'.$address['phone_mobile'].'</option>';
			}
			$mobilephones[$address['phone_mobile']] = $address['phone_mobile'];
			$allCustomerMobiles[] = $address['phone_mobile'];
		}

		$customer = new Customer((int)$params['id_customer']);
		$gender = new Gender((int)$customer->id_gender, $this->context->language->id);
		$vars = array(
			'{gender}' => ($gender->name == 'آقا') ? 'آقای' : $gender->name,
			'{firstname}' => $this->sanitizeURL($customer->firstname),
			'{lastname}' => $this->sanitizeURL($customer->lastname),
			'{email}' => $customer->email,
			'{shop_name}' => $this->shop,
		);
		$preparedsmstexts = str_replace(array_keys($vars), array_values($vars), $this->preparedSMSoptions());
		if(count($allCustomerMobiles)) $userLastSms = $this->getMobileLogs(0, $onpage = 15, $bulk = null, implode(',',$allCustomerMobiles)); else $userLastSms = array();
		$userLastSmsOutput = '';
		if (!class_exists('jDateTime')) {
			include_once('include/jDateTime.php');
		}
		foreach($userLastSms as $sms){
			$userLastSmsOutput .="<tr>
							<td>".$sms['bulk']."</td>
							<td>".$sms['phone']."</td>
							<td>".jDateTime::date('Y/m/d H:i:s', $sms['timestamp'], null, true,null)."</td>
							<td>".$sms['description']."</td>
							<td>".$this->l($sms['status'])."</td>
							<td>".$this->l($sms['delivery'])."</td>
						</tr>";
		}
		$this->smarty->assign(array(
			'preparedsmstexts' => $preparedsmstexts,
			'phone' => $mobilephones,
			'phoneoptions' => $mobilesoptions,
			'shop_id' => $this->shop_id,
			'customer_id' => $params['id_customer'],
			'userLastSms' => $userLastSmsOutput,
		));
		return $this->display(__FILE__, 'views/templates/admin/admincustomer.tpl');
	}	
	
	/**
	* ریدایرکت کاربر از لینک کوتاه محصول و صورتحساب به لینک کامل
	*/
	public function hookModuleRoutes() {
		$shortLinkKey = Configuration::get($this->prefix .'SHORTINVOICEKEY' , null , null , $this->shop_id);
		$shortProductKey = Configuration::get($this->prefix .'SHORTPRODUCTKEY' , null , null , $this->shop_id);
		return array(
			'module-ippanelsms-shortlink' => array(
				'controller' => 'shortlink',
				'rule' => $shortLinkKey.'/{order_reference}', //the desired page URL contains parameters
				'keywords' => array( //in the Keywords section we describe every parameter
					'order_reference' => array('regexp' => '[\w]+', 'param' => 'order_reference'), 
				),
				'params' => array(
					'fc' => 'module',
					'module' => $this->name,
				)
			),
			'module-ippanelsms-adminshortlink' => array(
				'controller' => 'adminshortlink',
				'rule' => 'ipf/{order_reference}', //the desired page URL contains parameters
				'keywords' => array( //in the Keywords section we describe every parameter
					'order_reference' => array('regexp' => '[\w]+', 'param' => 'order_reference'), 
				),
				'params' => array(
					'fc' => 'module',
					'module' => $this->name,
				)
			),
			'module-ippanelsms-productshortlink' => array(
				'controller' => 'productshortlink',
				'rule' => $shortProductKey.'/{product_id}', //the desired page URL contains parameters
				'keywords' => array( //in the Keywords section we describe every parameter
					'product_id' => array('regexp' => '[\w]+', 'param' => 'product_id'), 
				),
				'params' => array(
					'fc' => 'module',
					'module' => $this->name,
				)
			),

		);
	}
	
	/**
	* حذف آثار لینک از عبارت
	*/
	public function sanitizeURL($string = null, $except = null)
	{
		$find = array('http','https',':','//','.','www');
		if($except && isset($find[$except])) unset($find[$except]);
		$output = str_ireplace($find,array(''),array($string));
		return $output[0];
	}
}

/**
* کلاس api برای استفاده در افزونه های دیگر
*/
class IppanelSmsApi 
{
	public $api_version;
	public $help;
	public $message;
	public $reciver;
	public $position;
	public $pattern;
	public $tokens;
	public function __construct()
	{
		$this->version = '2.0.4';
		$this->help = '<p>برای استفاده از api افزونه پیامک پرستاشاپ ippanel کافی است دستور زیر را در محل مناسب از کدهای قالب یا ماژول خود قرار دهید.<br>در زیر 2 نمونه از ارسال پیامک معمولی و 2 نمونه از ارسال پیامک بر اساس الگو نمایش داده شده است. لازم به ذکر است که پیامک ارسال شده بر اساس الگو با سرعت بالا ارسال می شود. لیست الگو ها را می توانید در سامانه پیامک مشاهده نمایید. الگوهای مورد نیاز را نیز می توانید با ارسال تیکت در سامانه پیامک درخواست نمایید.</p>
		
		<pre dir="ltr">
if(class_exists("IppanelSmsApi")) { // بررسی اینکه آیا افزونه پیامک وجود دارد
	//برای ارسال پیامک معمولی دو روش وجود دارد:
	// 1- با استفاده از متغیرهای شیء گرا
	$smsObject = new IppanelSmsApi;
	$smsObject->message = "متن پیامک";
	$smsObject->reciver = "شماره همراه";
	$smsObject->position = "موقعیت ارسال پیامک";
	$result = $smsObject->send();					//Response: true/false
	
	//2- با استفاده از متغیرهای معمولی
	$smsObject = new IppanelSmsApi;
	$message = "متن پیامک";
	$reciver = "شماره همراه";
	$position = "موقعیت ارسال پیامک";
	$result = $smsObject->send($message,$reciver,$position);		//Response: true/false
	
	//برای ارسال پیامک ّبر اساس الگو دو روش وجود دارد:
	// 1- با استفاده از متغیرهای شیء گرا
	$smsObject = new IppanelSmsApi;
	$smsObject->pattern = "شماره الگو"; 			// string or number
	$smsObject->reciver = "شماره همراه"; 			// string or number
	$smsObject->position = "موقعیت ارسال پیامک"; 	// string
	$smsObject->tokens = array(						//مثال ها برای الگوی 142
		"firstname"=> "نام مشتری",
		"lastname"=> "نام خانوادگی مشتری",
		"site"=> "نام فروشگاه",
	);
	$result = $smsObject->sendPattern();			//response: true/false
	
	//2- با استفاده از متغیرهای معمولی
	$smsObject = new IppanelSmsApi;
	$pattern = "شماره الگو"; 			// string or number
	$reciver = "شماره همراه"; 			// string or number
	$position = "موقعیت ارسال پیامک"; 	// string
	$tokens = array(					//مثال ها برای الگوی 142
		"firstname"=> "نام مشتری",
		"lastname"=> "نام خانوادگی مشتری",
		"site"=> "نام فروشگاه",
	);
	$result = $smsObject->sendPattern($pattern,$reciver,$position,$tokens);			//response: true/false
}else{
	error_log("افزونه پیامک یافت نشد");
}
		</pre>
		
		<p>$result در دستور فوق در صورت ارسال شدن پیامک مقدار true و در صورت عدم موفقیت در ارسال پیامک مقدار false را برمیگرداند.</p>';
	}
	/** public function to send sms in Api
	* @param string $message: sms massage
	* @param string $reciver: recivers delimated by ;
	* @param string $position: event or position 
	* @return boolean
	*/
	public function send($message = null, $reciver = null, $position = null)
	{
		if($message) $this->message = $message;
		if($reciver) $this->reciver = $reciver;
		if($position) $this->position = $position;
		$ippanelsms = new IppanelSmsApi;
		$result = $ippanelsms->sendSMS( array( 'message' => $this->message, 'reciver' => $this->reciver ) ,$this->position );
		return $result;
	}
	/** public function to send pattetn sms in Api
	* @param string $pattern: Pattern ID
	* @param string $reciver: recivers delimated by ;
	* @param string $position: event or position 
	* @param string $tokens: array of token => value according to pattern 
	* @return boolean
	*/
	public function sendPattern($pattern = null, $reciver = null, $position = null, $tokens = null)
	{
		if($pattern) $this->pattern = $pattern;
		if($reciver) $this->reciver = $reciver;
		if($position) $this->position = $position;
		if($tokens) $this->tokens = $tokens;
		$messageArray = array();
		$messageArray[] = 'pcode:'.$this->pattern;
		if(is_array($this->tokens)){
			foreach($this->tokens as $key=>$value){
				$messageArray[] = $key.':'.$value;
			}
		}else{
			return false;
		}
		$this->message = implode(';',$messageArray);
		$ippanelsms = new IppanelSms;
		$result = $ippanelsms->sendSMS( array( 'message' => $this->message, 'reciver' => $this->reciver ) ,$this->position );
		return $result;
	}
}
/********TRANSLATION GATE***
$this->l('* You should add a mobile number for admin at Gateway Settings page to recieve admin alerts.')
$this->l('ACTIONS')
$this->l('Actions')
$this->l('Active fields in report:')
$this->l('Admin Invoice Url Pattern:')
$this->l('Admin invoice url pattern. You can use these variabels: ')
$this->l('Admin phone:')
$this->l('After 1 hour')
$this->l('After 15 Minutes')
$this->l('After 2 hour')
$this->l('After 24 hour')
$this->l('After 30 Minutes')
$this->l('After 5 Minutes')
$this->l('After 5 hour')
$this->l('Alert on create new account to admin:')
$this->l('Alert on create new account to customer:')
$this->l('Alert on create new address to customer:')
$this->l('Alert on set order tracking number to customer:')
$this->l('Alert on update order status to customer:')
$this->l('Alerts Settings')
$this->l('Alerts on new free order to admin:')
$this->l('Alerts on new free order to customer:')
$this->l('Alerts on new order to Admin:')
$this->l('Alerts on new order to Customer:')
$this->l('Alerts when pruducts goes out of stock:')
$this->l('All Customers SMS')
$this->l('Are you sure you want to delete log(s)?')
$this->l('BULK')
$this->l('Between Time:')
$this->l('By sample SMS will send from this number')
$this->l('CUSTOMER')
$this->l('Change it by support agent advice!')
$this->l('Choose Invoice short url first argument; for example, if you type "L" your short invoice link will be http://yoursiteurl/L/ASDFVR . Attention: If you change this all generaterd short links will redirect to 404 page.')
$this->l('Clear All Queued SMS')
$this->l('Clear Logs on Uninstall:')
$this->l('Close')
$this->l('Credit')
$this->l('Custom Text or Pattern')
$this->l('Custom Text or Variables of Pattern:')
$this->l('Customer First Address Mobile')
$this->l('Customer Numbers')
$this->l('DELIVERY')
$this->l('DESCRIPTION')
$this->l('Dear {firstname} {lastname},~Welcome to our site.')
$this->l('Dear {firstname} {lastname},~Your order has created.~Order No: {order_id}~{shop_name}')
$this->l('Debug:')
$this->l('Delay Between two sending sms')
$this->l('Delay Time:')
$this->l('Delete selected SMS log')
$this->l('Duration')
$this->l('Edit')
$this->l('Enable SMS system:')
$this->l('Enable if you want to clear all SMS logs when you uninstall this module.')
$this->l('Enable if you want to cron to send SMS by cron job system.')
$this->l('Enable if you want to debug parts of the module.')
$this->l('Enable if you want you site to send SMS.')
$this->l('Gateway Settings')
$this->l('Gateway alert')
$this->l('Guest Invoice Url Pattern:')
$this->l('Guest invoice url pattern. Default url is : "/guest-tracking?order_reference={order_reference}&email={guest_email}"')
$this->l('How many SMS should be sent on cron.')
$this->l('ID')
$this->l('If the process of sending SMS is slow, you can use cron to send it after a while.')
$this->l('Invoice Short Url Pattern:')
$this->l('Ippanel SMS Module')
$this->l('Ippanel SMS login Password.')
$this->l('Items to send on cron:')
$this->l('Last sent SMS to the customer')
$this->l('Low Credit Alert:')
$this->l('Manual SMS')
$this->l('Mobile number not found!')
$this->l('Never')
$this->l('New Account SMS')
$this->l('New Address SMS')
$this->l('New Order SMS')
$this->l('New order at {shop_name}~Customer: {firstname} {lastname}~Order: {order_id}~Payment: {payment}~Total: {total_paid} {currency}')
$this->l('No')
$this->l('Normal SMS will send from this number')
$this->l('Number per once:')
$this->l('ORDER')
$this->l('Order Address Mobile')
$this->l('Order Mobile Number:')
$this->l('Out of Stock SMS')
$this->l('PHONE')
$this->l('POSITION')
$this->l('Panel Address:')
$this->l('Panel Expire')
$this->l('Panel State')
$this->l('Password:')
$this->l('Pattern Wizard')
$this->l('Prepared SMS')
$this->l('Prepared Texts')
$this->l('Quantity for which a product is considered out of stock.')
$this->l('Ippanel Top Menu:')
$this->l('Refresh')
$this->l('Remove')
$this->l('SHOP')
$this->l('SMS System Testing')
$this->l('SMS Texts')
$this->l('STATUS')
$this->l('Save')
$this->l('Select One of theme if you want.')
$this->l('Select a prepared Text.')
$this->l('Select which type you want to send to admin on new account event.')
$this->l('Select which type you want to send to admin on new order event.')
$this->l('Select which type you want to send to admin when a product goes out of stock.')
$this->l('Select which type you want to send to customer on new account event.')
$this->l('Select which type you want to send to customer on new address event.')
$this->l('Select which type you want to send to customer on new order event.')
$this->l('Select which type you want to send to customer on update order event.')
$this->l('Select which type you want to send to customer on update order tracking number.')
$this->l('Send Customers')
$this->l('Send SMS Manual')
$this->l('Send SMS')
$this->l('Send selected SMS again')
$this->l('Send selected messages?')
$this->l('Send sms to customer.')
$this->l('Send sms to customers')
$this->l('Send to Admin')
$this->l('Send to All Customers')
$this->l('Send your SMS in minutes?')
$this->l('Sender is empty')
$this->l('Sent SMS')
$this->l('Sent to:')
$this->l('Service Sms number:')
$this->l('Settings is not updated!')
$this->l('Settings updated.')
$this->l('Shop Name:')
$this->l('Shop name in SMS texts.')
$this->l('Show Rangine Top menu.')
$this->l('Show an alert on admin dashboard when your SMS credit is low.')
$this->l('Sms number:')
$this->l('TIME')
$this->l('Text for Admin on New Account:')
$this->l('Text for Admin on New Order:')
$this->l('Text for Admin on out of stock:')
$this->l('Text for Customer on New Account:')
$this->l('Text for Customer on New Address:')
$this->l('Text for Customer on New Order:')
$this->l('Text for Customer on Update Order Tracking:')
$this->l('Text for Customer on Update Order:')
$this->l('Text is empty')
$this->l('Text message:')
$this->l('Text')
$this->l('The customer phone.')
$this->l('The message is not saved!')
$this->l('The sms sent to customers.')
$this->l('The sms sent.')
$this->l('There is not any prepared text.')
$this->l('This phone will get all administrator SMS')
$this->l('Threshold:')
$this->l('To send multiple sms, saperate numbers with ; ex:09123456789;09101234567.')
$this->l('Type SMS text.')
$this->l('Type shop name if you want.')
$this->l('Type your SMS Panel pasword')
$this->l('Type your SMS Panel user name')
$this->l('Update Order SMS')
$this->l('Update Order Tracking SMS')
$this->l('Update Settings')
$this->l('Update Time')
$this->l('Use cron in this states:')
$this->l('Use cron to send SMS:')
$this->l('User Invoice Url Pattern:')
$this->l('User invoice url pattern. Default url is : "/index.php?controller=pdf-invoice&id_order={order_id}" ')
$this->l('Username:')
$this->l('Variables:')
$this->l('Welcome to Ippanel SMS module for Prestashop. You can have the best exprience of using SMS module to develope your business.')
$this->l('Welcome')
$this->l('When enabled, the checkbox in order status update pane of order edit page checked as default.')
$this->l('When enabled, the checkbox in order tracking number update pane of order edit page checked as default.')
$this->l('When you enable debug mode, the module will write some log in debug folder.')
$this->l('Which mobile number should be used for order alerts. If you have guest customer, choose order number')
$this->l('Which number should be sent at once. If you want to send all at once leave it empty')
$this->l('Which number should be used to send this sms. ex: +985000XXXXXXX')
$this->l('Which of these states run in Cron.')
$this->l('Yes')
$this->l('default')
$this->l('ex: +985000XXXXXXX')
$this->l('ex: 09123456789')
$this->l('ex: username.')
$this->l('https://ippanel.com')
$this->l('insert valid phone number')
$this->l('multiple')
$this->l('unique phone mobile(s) from customers is exist')
$this->l('{firstname} {lastname} registred on site')
$this->l('{product_id} goes out to stock on {shop_name}.~Quantity: {product_quantity}')
$this->l('Text Message:')
*/